@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -42,14 +42,8 @@
 static GtkWidget *
 box_pick_parent( GtkWidget *par )
 {
-	if( !par ) {
-		Mainw *mainw;
-
-		if( (mainw = mainw_pick_one()) )
-			return( GTK_WIDGET( mainw ) );
-		else
-			return( main_window_top );
-	}
+	if( !par ) 
+		return( GTK_WIDGET( mainw_pick_one() ) );
 	else 
 		return( par );
 }
@@ -148,7 +142,7 @@ box_alert( GtkWidget *par )
 {
 	char buf[MAX_DIALOG_TEXT];
 
-	if( !main_window_gdk ) {
+	if( main_option_batch ) {
 		/* No X, just print.
 		 */
 		fprintf( stderr, "%s\n", error_get_top() );
