@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -304,7 +304,7 @@ apply_image_call( Reduce *rc,
 
 	/* Try to load image from given string.
 	 */
-	if( !(fn = path_find_file( PATH_SEARCH, filename )) )
+	if( !(fn = path_find_file( filename )) )
 		reduce_throw( rc );
 
 	/* Reattach the mode and load.
@@ -520,7 +520,7 @@ apply_search_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 	PEPOINTRIGHT( arg[0], &rhs );
 	(void) reduce_get_string( rc, &rhs, buf, MAX_STRSIZE );
 
-	if( !(fn = path_find_file( PATH_SEARCH, buf )) )
+	if( !(fn = path_find_file( buf )) )
 		/* If not found, return [].
 		 */
 		fn = im_strdup( NULL, "" );
@@ -835,92 +835,130 @@ apply_vo_call_call( Reduce *rc,
 static BuiltinInfo builtin_table[] = {
 	/* Other.
 	 */
-	{ "dir", FALSE, 1, &dir_args[0], &apply_dir_call },
-	{ "search", FALSE, 1, &search_args[0], &apply_search_call },
-	{ "error", FALSE, 1, &error_args[0], &apply_error_call },
-	{ "print", FALSE, 1, &print_args[0], &apply_print_call },
-	{ "expand", FALSE, 1, &expand_args[0], &apply_expand_call },
-	{ "name2gtype", FALSE, 1, &name2gtype_args[0], &apply_name2gtype_call },
-	{ "gtype2name", FALSE, 1, &gtype2name_args[0], &apply_gtype2name_call },
-	{ "_", FALSE, 1, &underscore_args[0], &apply_underscore_call },
+	{ "dir", N_( "return list of names of members" ),
+		FALSE, 1, &dir_args[0], &apply_dir_call },
+	{ "search", N_( "search for file" ),
+		FALSE, 1, &search_args[0], &apply_search_call },
+	{ "error", N_( "raise error" ),
+		FALSE, 1, &error_args[0], &apply_error_call },
+	{ "print", N_( "convert to [char]" ),
+		FALSE, 1, &print_args[0], &apply_print_call },
+	{ "expand", N_( "filexpand environment variables" ),
+		FALSE, 1, &expand_args[0], &apply_expand_call },
+	{ "name2gtype", N_( "convert [char] to GType" ),
+		FALSE, 1, &name2gtype_args[0], &apply_name2gtype_call },
+	{ "gtype2name", N_( "convert GType to [char]" ),
+		FALSE, 1, &gtype2name_args[0], &apply_gtype2name_call },
+	{ "_", N_( "look up localised string" ),
+		FALSE, 1, &underscore_args[0], &apply_underscore_call },
 
 	/* vips8 wrapper.
 	 */
-	{ "vips_object_new", FALSE, IM_NUMBER( vo_new_args ), 
+	{ "vips_object_new", N_( "create new vips8 object" ),
+		FALSE, IM_NUMBER( vo_new_args ), 
 		&vo_new_args[0], apply_vo_new_call },
-	{ "vips_call", FALSE, IM_NUMBER( vo_call_args ), 
+	{ "vips_call", N_( "call vips8 operator" ),
+		FALSE, IM_NUMBER( vo_call_args ), 
 		&vo_call_args[0], apply_vo_call_call },
 
 	/* Predicates.
 	 */
-	{ "is_image", FALSE, IM_NUMBER( pred_args ), 
+	{ "is_image", N_( "true if argument is primitive image" ),
+		FALSE, IM_NUMBER( pred_args ), 
 		&pred_args[0], apply_pred_call },
-	{ "is_bool", FALSE, IM_NUMBER( pred_args ), 
+	{ "is_bool", N_( "true if argument is primitive bool" ),
+		FALSE, IM_NUMBER( pred_args ), 
 		&pred_args[0], apply_pred_call },
-	{ "is_real", FALSE, IM_NUMBER( pred_args ), 
+	{ "is_real", N_( "true if argument is primitive real number" ),
+		FALSE, IM_NUMBER( pred_args ), 
 		&pred_args[0], apply_pred_call },
-	{ "is_class", FALSE, IM_NUMBER( pred_args ), 
+	{ "is_class", N_( "true if argument is class" ),
+		FALSE, IM_NUMBER( pred_args ), 
 		&pred_args[0], apply_pred_call },
-	{ "is_char", FALSE, IM_NUMBER( pred_args ), 
+	{ "is_char", N_( "true if argument is primitive char" ),
+		FALSE, IM_NUMBER( pred_args ), 
 		&pred_args[0], apply_pred_call },
-	{ "is_list", FALSE, IM_NUMBER( pred_args ), 
+	{ "is_list", N_( "true if argument is primitive list" ),
+		FALSE, IM_NUMBER( pred_args ), 
 		&pred_args[0], apply_pred_call },
-	{ "is_complex", FALSE, IM_NUMBER( pred_args ), 
+	{ "is_complex", N_( "true if argument is primitive complex" ),
+		FALSE, IM_NUMBER( pred_args ), 
 		&pred_args[0], apply_pred_call },
-	{ "is_instanceof", FALSE, IM_NUMBER( is_instanceof_args ), 
+	{ "is_instanceof", N_( "true if argument class instance of type" ),
+		FALSE, IM_NUMBER( is_instanceof_args ), 
 		&is_instanceof_args[0], apply_is_instanceof_call },
-	{ "has_member", FALSE, IM_NUMBER( has_member_args ), 
+	{ "has_member", N_( "true if class has named member" ),
+		FALSE, IM_NUMBER( has_member_args ), 
 		&has_member_args[0], apply_has_member_call },
 
 	/* List and complex projections.
 	 */
-	{ "re", TRUE, IM_NUMBER( complex_args ), 
+	{ "re", N_( "real part of complex" ),
+		TRUE, IM_NUMBER( complex_args ), 
 		&complex_args[0], apply_complex_call },
-	{ "im", TRUE, IM_NUMBER( complex_args ), 
+	{ "im", N_( "imaginary part of complex" ),
+		TRUE, IM_NUMBER( complex_args ), 
 		&complex_args[0], apply_complex_call },
-	{ "hd", TRUE, IM_NUMBER( flist_args ), 
+	{ "hd", N_( "head of list" ),
+		TRUE, IM_NUMBER( flist_args ), 
 		&flist_args[0], apply_list_call },
-	{ "tl", TRUE, IM_NUMBER( flist_args ), 
+	{ "tl", N_( "tail of list" ),
+		TRUE, IM_NUMBER( flist_args ), 
 		&flist_args[0], apply_list_call },
 
 	/* Math.
 	 */
-	{ "sin", TRUE, IM_NUMBER( math_args ), 
+	{ "sin", N_( "sine of real number" ),
+		TRUE, IM_NUMBER( math_args ), 
 		&math_args[0], apply_math_call },
-	{ "cos", TRUE, IM_NUMBER( math_args ), 
+	{ "cos", N_( "cosine of real number" ),
+		TRUE, IM_NUMBER( math_args ), 
 		&math_args[0], apply_math_call },
-	{ "tan", TRUE, IM_NUMBER( math_args ), 
+	{ "tan", N_( "tangent of real number" ),
+		TRUE, IM_NUMBER( math_args ), 
 		&math_args[0], apply_math_call }, 
-	{ "asin", TRUE, IM_NUMBER( math_args ), 
+	{ "asin", N_( "arc sine of real number" ),
+		TRUE, IM_NUMBER( math_args ), 
 		&math_args[0], apply_math_call },
-	{ "acos", TRUE, IM_NUMBER( math_args ), 
+	{ "acos", N_( "arc cosine of real number" ),
+		TRUE, IM_NUMBER( math_args ), 
 		&math_args[0], apply_math_call },
-	{ "atan", TRUE, IM_NUMBER( math_args ), 
+	{ "atan", N_( "arc tangent of real number" ),
+		TRUE, IM_NUMBER( math_args ), 
 		&math_args[0], apply_math_call }, 
-	{ "log", TRUE, IM_NUMBER( math_args ), 
+	{ "log", N_( "log base e of real number" ),
+		TRUE, IM_NUMBER( math_args ), 
 		&math_args[0], apply_math_call },
-	{ "log10", TRUE, IM_NUMBER( math_args ), 
+	{ "log10", N_( "log base 10 of real number" ),
+		TRUE, IM_NUMBER( math_args ), 
 		&math_args[0], apply_math_call },
-	{ "exp", TRUE, IM_NUMBER( math_args ), 
+	{ "exp", N_( "e to the power of real number" ),
+		TRUE, IM_NUMBER( math_args ), 
 		&math_args[0], apply_math_call },
-	{ "exp10", TRUE, IM_NUMBER( math_args ), 
+	{ "exp10", N_( "10 to the power of real number" ),
+		TRUE, IM_NUMBER( math_args ), 
 		&math_args[0], apply_math_call },
-	{ "ceil", TRUE, IM_NUMBER( math_args ), 
+	{ "ceil", N_( "real to int, rounding up" ),
+		TRUE, IM_NUMBER( math_args ), 
 		&math_args[0], apply_math_call },
-	{ "floor", TRUE, IM_NUMBER( math_args ), 
+	{ "floor", N_( "real to int, rounding down" ),
+		TRUE, IM_NUMBER( math_args ), 
 		&math_args[0], apply_math_call },
 
 	/* Optional GSL funcs.
 	 */
-	{ "gammq", TRUE, IM_NUMBER( gammq_args ), 
+	{ "gammq", N_( "gamma function" ),
+		TRUE, IM_NUMBER( gammq_args ), 
 		&gammq_args[0], apply_gammq_call },
 
 	/* Constructors.
 	 */
-	{ "vips_image", FALSE,  
-		IM_NUMBER( image_args ), &image_args[0], apply_image_call },
-	{ "read", FALSE,  
-		IM_NUMBER( read_args ), &read_args[0], apply_read_call },
+	{ "vips_image", N_( "load vips image" ),
+		FALSE, IM_NUMBER( image_args ), 
+		&image_args[0], apply_image_call },
+	{ "read", N_( "load text file" ),
+		FALSE, IM_NUMBER( read_args ), 
+		&read_args[0], apply_read_call },
 };
 
 #ifdef HAVE_GSL
