@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -243,7 +243,7 @@ call_usage( VipsBuf *buf, im_function *fn )
 static void
 call_error_arg( CallInfo *vi, HeapNode **arg, int argi )
 {
-	char txt[1000];
+	char txt[10000];
 	VipsBuf buf = VIPS_BUF_STATIC( txt );
 
 	error_top( _( "Bad argument." ) );
