@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -127,7 +127,6 @@ classmodel_graphic_save_cb( iWindow *iwnd,
 	if( (filename = filesel_get_filename( filesel )) ) {
 		if( class->graphic_save( classmodel, 
 			GTK_WIDGET( iwnd ), filename ) ) {
-			path_add_file( filename );
 			IM_SETSTR( classmodel->filename, filename );
 			iobject_changed( IOBJECT( classmodel ) );
 
@@ -153,7 +152,7 @@ classmodel_graphic_save( Classmodel *classmodel, GtkWidget *parent )
 	if( !class->graphic_save ) {
 		error_top( _( "Not implemented." ) );
 		error_sub( _( "_%s() method not implemented for %s." ), 
-			"graphic_save", G_OBJECT_TYPE_NAME( classmodel ) );
+			"graphic_save", IOBJECT_GET_CLASS_NAME( classmodel ) );
 		iwindow_alert( parent, GTK_MESSAGE_ERROR );
 		return;
 	}
@@ -161,7 +160,7 @@ classmodel_graphic_save( Classmodel *classmodel, GtkWidget *parent )
 	filesel = filesel_new();
 	row_qualified_name( HEAPMODEL( classmodel )->row, &buf );
 	iwindow_set_title( IWINDOW( filesel ), _( "Save %s \"%s\"" ), 
-		G_OBJECT_TYPE_NAME( classmodel ), vips_buf_all( &buf ) );
+		IOBJECT_GET_CLASS_NAME( classmodel ), vips_buf_all( &buf ) );
 	filesel_set_flags( FILESEL( filesel ), TRUE, TRUE );
 	filesel_set_filetype( FILESEL( filesel ), 
 		class->filetype, 
@@ -198,7 +197,6 @@ classmodel_graphic_replace_cb( iWindow *iwnd,
 			g_object_ref( G_OBJECT( classmodel ) );
 
 			symbol_recalculate_all();
-			path_add_file( filename );
 			IM_SETSTR( classmodel->filename, filename );
 			iobject_changed( IOBJECT( classmodel ) );
 
@@ -227,7 +225,7 @@ classmodel_graphic_replace( Classmodel *classmodel, GtkWidget *parent )
 		error_top( _( "Not implemented." ) );
 		error_sub( _( "_%s() method not implemented for %s." ), 
 			"graphic_replace",
-			G_OBJECT_TYPE_NAME( classmodel ) );
+			IOBJECT_GET_CLASS_NAME( classmodel ) );
 		iwindow_alert( parent, GTK_MESSAGE_ERROR );
 		return;
 	}
@@ -235,7 +233,7 @@ classmodel_graphic_replace( Classmodel *classmodel, GtkWidget *parent )
 	row_qualified_name( HEAPMODEL( classmodel )->row, &buf );
 	filesel = filesel_new();
 	iwindow_set_title( IWINDOW( filesel ), _( "Replace %s \"%s\"" ), 
-		G_OBJECT_TYPE_NAME( classmodel ), vips_buf_all( &buf ) );
+		IOBJECT_GET_CLASS_NAME( classmodel ), vips_buf_all( &buf ) );
 	filesel_set_flags( FILESEL( filesel ), TRUE, FALSE );
 	filesel_set_filetype( FILESEL( filesel ), 
 		class->filetype, 
@@ -314,7 +312,9 @@ classmodel_iimage_expr_model( Model *model, ClassmodelSearch *parms )
 	 * derived from Image, display on the derived class, not on the
 	 * superclass.
 	 */
-	if( IS_IIMAGE( model ) && !is_super( HEAPMODEL( model )->row->sym ) &&
+	if( IS_IIMAGE( model ) && 
+		HEAPMODEL( model )->row->sym &&
+		!is_super( HEAPMODEL( model )->row->sym ) &&
 		!is_this( HEAPMODEL( model )->row->sym ) ) {
 		iImage *iimage = IIMAGE( model );
 
@@ -683,6 +683,8 @@ classmodel_buildedit_member( Classmodel *classmodel,
 {
 	GtkWidget *widget;
 
+	widget = NULL; 
+
 	switch( m->type ) {
 	case CLASSMODEL_MEMBER_INT:
 	case CLASSMODEL_MEMBER_ENUM:
@@ -750,28 +752,27 @@ classmodel_edit( GtkWidget *parent, Model *model )
 	ClassmodelClass *class = CLASSMODEL_GET_CLASS( classmodel );
 
 	if( class->n_members ) {
-		Row *row = HEAPMODEL( classmodel )->row;
 		GtkWidget *idlg;
 		ClassmodelEdit *eds = INEW( NULL, ClassmodelEdit );
 
 		eds->classmodel = classmodel;
 
 		idlg = idialog_new();
-		/* Expands to "Edit Toggle A1".
+		/* Expands to eg. "Edit Toggle A1".
 		 */
 		iwindow_set_title( IWINDOW( idlg ), _( "Edit %s %s" ),
-			G_OBJECT_TYPE_NAME( classmodel ),
-			IOBJECT( row )->name );
+			IOBJECT_GET_CLASS_NAME( model ),
+			IOBJECT( HEAPMODEL( model )->row )->name );
 		idialog_set_build( IDIALOG( idlg ), 
 			(iWindowBuildFn) classmodel_buildedit, eds, 
 			NULL, NULL );
 		idialog_set_callbacks( IDIALOG( idlg ), 
 			iwindow_true_cb, NULL, idialog_free_client, eds );
-		/* Expands to "Set Toggle".
+		/* Expands to eg. "Set Toggle".
 		 */
 		idialog_add_ok( IDIALOG( idlg ), 
 			classmodel_done_cb, _( "Set %s" ), 
-			G_OBJECT_TYPE_NAME( classmodel ) );
+			IOBJECT_GET_CLASS_NAME( classmodel ) );
 		iwindow_set_parent( IWINDOW( idlg ), parent );
 		idialog_set_iobject( IDIALOG( idlg ), IOBJECT( classmodel ) );
 		iwindow_build( IWINDOW( idlg ) );
@@ -789,7 +790,7 @@ classmodel_save_member( Classmodel *classmodel,
 	switch( m->type ) {
 	case CLASSMODEL_MEMBER_INT:
 	case CLASSMODEL_MEMBER_ENUM:
-		if( !set_prop( xthis, m->save_name, "%d", 
+		if( !set_iprop( xthis, m->save_name, 
 			G_STRUCT_MEMBER( int, classmodel, m->offset ) ) )
 			return( FALSE );
 		break;
@@ -807,7 +808,7 @@ classmodel_save_member( Classmodel *classmodel,
 		break;
 
 	case CLASSMODEL_MEMBER_STRING:
-		if( !set_prop( xthis, m->save_name, "%s", 
+		if( !set_sprop( xthis, m->save_name, 
 			G_STRUCT_MEMBER( char *, classmodel, m->offset ) ) )
 			return( FALSE );
 		break;
@@ -836,8 +837,8 @@ classmodel_save_member( Classmodel *classmodel,
 		const int n = value->width * value->height;
 
 		if( !set_dlprop( xthis, "value", value->coeff, n ) ||
-			!set_prop( xthis, "width", "%d", value->width ) ||
-			!set_prop( xthis, "height", "%d", value->height ) )
+			!set_iprop( xthis, "width", value->width ) ||
+			!set_iprop( xthis, "height", value->height ) )
 			return( FALSE );
 
 		break;
@@ -1443,7 +1444,7 @@ classmodel_update( Classmodel *classmodel )
 	 */
 	classmodel_set_edited( classmodel, TRUE );
 	expr_dirty( row->expr, link_serial_new() );
-	filemodel_set_modified( FILEMODEL( row->ws ), TRUE );
+	workspace_set_modified( row->ws, TRUE );
 }
 
 /* Make a new classmodel subtype (eg. TYPE_PATHNAME) and link it on.
