@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -2113,7 +2113,7 @@ compile_copy_sym( Symbol *sym, Compile *dest )
 		break;
 
 	case SYM_WORKSPACE:
-	case SYM_WORKSPACEGROUP:
+	case SYM_WORKSPACEROOT:
 	case SYM_ROOT:
 	case SYM_EXTERNAL:
 	case SYM_BUILTIN:
