@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -467,8 +467,6 @@ im_lintrauc( double a, IMAGE *in, double b, IMAGE *out )
 { 
         LintraInfo *inf;
 
-        /* Check args.
-         */
         if( in->Coding != IM_CODING_NONE ) {
                 im_error( "im_lintrauc", _( "not uncoded" ) );
                 return( -1 );
@@ -479,8 +477,6 @@ im_lintrauc( double a, IMAGE *in, double b, IMAGE *out )
 	out->Bbits = IM_BBITS_BYTE;
 	out->BandFmt = IM_BANDFMT_UCHAR;
 
-        /* Make space for a little buffer.
-         */
         if( !(inf = IM_NEW( out, LintraInfo )) )
                 return( -1 );
 	inf->a = a;
@@ -488,8 +484,6 @@ im_lintrauc( double a, IMAGE *in, double b, IMAGE *out )
 	inf->in = in;
 	inf->out = out;
 
-        /* Generate!
-         */
 	if( im_wrapone( in, out,
 		(im_wrapone_fn) lintrauc_gen, in, inf ) )
 		return( -1 );
@@ -506,11 +500,12 @@ conversion_make_repaint( Conversion *conv, IMAGE *in )
 
 	/* 7 is sRGB.
 
-		FIXME ... should let other displays be used here, see
-		../scraps/calibrate.[hc]
+		this is all deprecated and unused now with vips-7.31 and later
+		tag as unused to stop gcc complaints
 
 	 */
-	struct im_col_display *display = im_col_displays( 7 );
+	struct im_col_display *display __attribute__ ((unused)) = 
+		im_col_displays( 7 );
 
 	/* Do we do colorimetric type conversions? Look for
 	 * interpret-type-toggle.
@@ -615,6 +610,7 @@ conversion_make_repaint( Conversion *conv, IMAGE *in )
 		(in->Type == IM_TYPE_LCH ||
 		in->Type == IM_TYPE_YXY ||
 		in->Type == IM_TYPE_UCS ||
+		in->Type == VIPS_INTERPRETATION_scRGB ||
 		in->Type == IM_TYPE_LAB ||
 		in->Type == IM_TYPE_XYZ) ) {
 		IMAGE *t[2];
@@ -646,6 +642,18 @@ conversion_make_repaint( Conversion *conv, IMAGE *in )
 			in = t[1];
 		}
 
+		if( in->Type == VIPS_INTERPRETATION_scRGB ) {
+			VipsImage *x;
+
+			if( vips_scRGB2sRGB( in, &x, NULL ) ) {
+				im_close( out );
+				return( NULL );
+			}
+			vips_object_local( out, x ); 
+
+			in = x;
+		}
+
 		if( in->Type == IM_TYPE_YXY ) {
 			if( im_open_local_array( out, t, 2, "conv-1", "p" ) ||
 				im_clip2fmt( in, t[0], IM_BANDFMT_FLOAT ) ||
@@ -701,8 +709,9 @@ conversion_make_repaint( Conversion *conv, IMAGE *in )
 			(conv->scale != 1.0 || conv->offset != 0.0) ) {
 			IMAGE *t = im_open_local( out, "conv:1", "p" );
 
-			if( !t || im_lintrauc( conv->scale, in, 
-				conv->offset, t ) ) {
+			if( !t || 
+				im_lintrauc( conv->scale, in, 
+					conv->offset, t ) ) {
 				im_close( out );
 				return( NULL );
 			}
