@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -27,7 +27,7 @@
 
 */
 
-typedef void (*DoubleclickFunc)( GtkWidget *, void *, guint );
+typedef void (*DoubleclickFunc)( GtkWidget *, GdkEvent *, void * );
 #define DOUBLECLICK_FUNC( fn ) ((DoubleclickFunc) (fn))
 
 void doubleclick_add( GtkWidget *wid, gboolean dsingle,
