@@ -16,9 +16,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -122,6 +122,8 @@ filemodel_set_modified( Filemodel *filemodel, gboolean modified )
 void 
 filemodel_set_window_hint( Filemodel *filemodel, iWindow *iwnd )
 {
+	/* This can be called repeatedly if objects are moved between windows.
+	 */
 	filemodel->window_hint = iwnd;
 }
 
@@ -131,20 +133,42 @@ filemodel_get_window_hint( Filemodel *filemodel )
 	if( filemodel->window_hint )
 		return( filemodel->window_hint );
 	else
-		return( IWINDOW( main_window_top ) );
+		return( IWINDOW( mainw_pick_one() ) );
 }
 
 gboolean
-filemodel_save_all( Filemodel *filemodel, const char *filename )
+filemodel_top_save( Filemodel *filemodel, const char *filename )
 {
 	FilemodelClass *filemodel_class = FILEMODEL_GET_CLASS( filemodel );
 
-	if( filemodel_class->save_all ) 
-		return( filemodel_class->save_all( filemodel, filename ) );
+	if( filemodel_class->top_save ) {
+		char *old_filename; 
+		int result;
+
+		/* We must always have the new filename in the save file or
+		 * auto path rewriting will get confused on reload.
+		 *
+		 * Equally, we must not change the filename on the model, in
+		 * case this save is not something initiated by the user, for
+		 * example, an auto-backup of the workspace.
+		 *
+		 * Save and restore the filename. Our caller must set the
+		 * final filename, if required (after save-as, for example).
+		 */
+		old_filename = g_strdup( filemodel->filename ); 
+		filemodel_set_filename( filemodel, filename );
+
+		result = filemodel_class->top_save( filemodel, filename );
+
+		filemodel_set_filename( filemodel, old_filename );
+		g_free( old_filename );
+
+		return( result ); 
+	}
 	else {
 		error_top( _( "Not implemented." ) );
 		error_sub( _( "_%s() not implemented for class \"%s\"." ), 
-			"save_all",
+			"top_save",
 			G_OBJECT_CLASS_NAME( filemodel_class ) );
 
 		return( FALSE );
@@ -158,7 +182,8 @@ filemodel_info( iObject *iobject, VipsBuf *buf )
 
 	IOBJECT_CLASS( parent_class )->info( iobject, buf );
 
-	vips_buf_appendf( buf, "filename = \"%s\"\n", NN( filemodel->filename ) );
+	vips_buf_appendf( buf, "filename = \"%s\"\n", 
+		NN( filemodel->filename ) );
 	vips_buf_appendf( buf, "modified = \"%s\"\n", 
 		bool_to_char( filemodel->modified ) );
 	vips_buf_appendf( buf, "registered = \"%s\"\n", 
@@ -173,6 +198,17 @@ void
 filemodel_set_filename( Filemodel *filemodel, const char *filename )
 {
 	if( filemodel->filename != filename ) {
+		char buf[FILENAME_MAX];
+
+		/* We want to keep the absolute, compact form of the filename 
+		 * inside the object so we don't get a dependency on CWD.
+		 */
+		if( filename ) {
+			im_strncpy( buf, filename, FILENAME_MAX );
+			path_compact( buf );
+			filename = buf;
+		}
+
 		IM_SETSTR( filemodel->filename, filename );
 		iobject_changed( IOBJECT( filemodel ) );
 	}
@@ -196,11 +232,32 @@ filemodel_finalize( GObject *gobject )
 #endif /*DEBUG*/
 
 	IM_FREE( filemodel->filename );
-	filemodel_unregister( filemodel );
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 }
 
+static void
+filemodel_dispose( GObject *gobject )
+{
+	Filemodel *filemodel;
+
+	g_return_if_fail( gobject != NULL );
+	g_return_if_fail( IS_FILEMODEL( gobject ) );
+
+	filemodel = FILEMODEL( gobject );
+
+#ifdef DEBUG
+	printf( "filemodel_dispose: %s \"%s\" (%s)\n", 
+		G_OBJECT_TYPE_NAME( filemodel ), 
+		NN( IOBJECT( filemodel )->name ),
+		NN( filemodel->filename ) );
+#endif /*DEBUG*/
+
+	filemodel_unregister( filemodel );
+
+	G_OBJECT_CLASS( parent_class )->dispose( gobject );
+}
+
 static xmlNode *
 filemodel_save( Model *model, xmlNode *xnode )
 {
@@ -261,7 +318,7 @@ filemodel_real_set_modified( Filemodel *filemodel, gboolean modified )
 /* Save to filemodel->filename.
  */
 static gboolean
-filemodel_save_all_xml( Filemodel *filemodel, const char *filename )
+filemodel_top_save_xml( Filemodel *filemodel, const char *filename )
 {
 	xmlDoc *xdoc;
 	char namespace[256];
@@ -301,7 +358,7 @@ filemodel_save_all_xml( Filemodel *filemodel, const char *filename )
 			filename, xdoc, NULL, NULL ) == -1 ) {
 		error_top( _( "Save failed." ) );
 		error_sub( _( "Save of %s \"%s\" to file \"%s\" failed.\n%s" ),
-			G_OBJECT_TYPE_NAME( filemodel ), 
+			IOBJECT_GET_CLASS_NAME( filemodel ), 
 			NN( IOBJECT( filemodel )->name ),
 			NN( filename ),
 			g_strerror( errno ) );
@@ -316,7 +373,7 @@ filemodel_save_all_xml( Filemodel *filemodel, const char *filename )
 }
 
 static gboolean
-filemodel_save_all_text( Filemodel *filemodel, const char *filename )
+filemodel_top_save_text( Filemodel *filemodel, const char *filename )
 {
 	iOpenFile *of;
 
@@ -334,33 +391,31 @@ filemodel_save_all_text( Filemodel *filemodel, const char *filename )
 }
 
 static gboolean
-filemodel_real_save_all( Filemodel *filemodel, const char *filename )
+filemodel_real_top_save( Filemodel *filemodel, const char *filename )
 {
 	ModelClass *model_class = MODEL_GET_CLASS( filemodel );
 
 #ifdef DEBUG
-	printf( "filemodel_real_save_all: save %s \"%s\" to file \"%s\"\n", 
+	printf( "filemodel_real_top_save: save %s \"%s\" to file \"%s\"\n", 
 		G_OBJECT_TYPE_NAME( filemodel ), 
 		NN( IOBJECT( filemodel )->name ),
 		filename );
 #endif /*DEBUG*/
 
 	if( model_class->save_text ) {
-		if( !filemodel_save_all_text( filemodel, filename ) )
+		if( !filemodel_top_save_text( filemodel, filename ) )
 			return( FALSE );
 	}
 	else if( model_class->save ) {
-		if( !filemodel_save_all_xml( filemodel, filename ) )
+		if( !filemodel_top_save_xml( filemodel, filename ) )
 			return( FALSE );
 	}
 	else {
 		error_top( _( "Not implemented." ) );
-		error_sub( _( "filemodel_real_save_all: no save method" ) );
+		error_sub( _( "filemodel_real_top_save: no save method" ) );
 		return( FALSE );
 	}
 
-	path_add_file( filename );
-
 	return( TRUE );
 }
 
@@ -374,6 +429,7 @@ filemodel_class_init( FilemodelClass *class )
 	parent_class = g_type_class_peek_parent( class );
 
 	gobject_class->finalize = filemodel_finalize;
+	gobject_class->dispose = filemodel_dispose;
 
 	iobject_class->info = filemodel_info;
 
@@ -382,7 +438,7 @@ filemodel_class_init( FilemodelClass *class )
 	
 	class->top_load = filemodel_real_top_load;
 	class->set_modified = filemodel_real_set_modified;
-	class->save_all = filemodel_real_save_all;
+	class->top_save = filemodel_real_top_save;
 
 	/* NULL isn't an allowed value -- this gets overridden by our
 	 * subclasses.
@@ -456,10 +512,7 @@ static gboolean
 filemodel_load_all_xml( Filemodel *filemodel, 
 	Model *parent, ModelLoadState *state )
 {
-	const char *tname = G_OBJECT_TYPE_NAME( filemodel );
-
 	xmlNode *xnode;
-	xmlNode *xstart;
 
 	/* Check the root element for type/version compatibility.
 	 */
@@ -487,29 +540,19 @@ filemodel_load_all_xml( Filemodel *filemodel,
 		state->major, state->minor, state->micro );
 #endif /*DEBUG*/
 
-	if( !(xstart = get_node( xnode, tname )) ) {
-		error_top( _( "Load failed." ) );
-		error_sub( _( "Can't load XML file \"%s\", "
-			"the file does not contain a %s." ), 
-			state->filename, tname );
-		return( FALSE );
-	}
-
-	/* Set the global loadstate so the lexer can see it.
-	 */
-	if( filemodel_top_load( filemodel, state, parent, xstart ) ) 
+	if( filemodel_top_load( filemodel, state, parent, xnode ) ) 
 		return( FALSE );
 
 	return( TRUE );
 }
 
 static gboolean
-filemodel_load_all_xml_file( Filemodel *filemodel, 
-	Model *parent, const char *filename )
+filemodel_load_all_xml_file( Filemodel *filemodel, Model *parent, 
+	const char *filename, const char *filename_user )
 {
 	ModelLoadState *state;
 
-	if( !(state = model_loadstate_new( filename )) )
+	if( !(state = model_loadstate_new( filename, filename_user )) )
 		return( FALSE );
 	if( !filemodel_load_all_xml( filemodel, parent, state ) ) {
 		model_loadstate_destroy( state );
@@ -538,8 +581,8 @@ filemodel_load_all_xml_openfile( Filemodel *filemodel,
 }
 
 static gboolean
-filemodel_load_all_text( Filemodel *filemodel, 
-	Model *parent, const char *filename )
+filemodel_load_all_text( Filemodel *filemodel, Model *parent, 
+	const char *filename, const char *filename_user )
 {
 	iOpenFile *of;
 
@@ -556,9 +599,13 @@ filemodel_load_all_text( Filemodel *filemodel,
 }
 
 /* Load filename into filemodel ... can mean merge as well as init.
+ *
+ * We load from @filename. If @filename_user is non-NULL, that's the filename
+ * we should record in the model.
  */
 gboolean
-filemodel_load_all( Filemodel *filemodel, Model *parent, const char *filename )
+filemodel_load_all( Filemodel *filemodel, Model *parent, 
+	const char *filename, const char *filename_user )
 {
 	ModelClass *model_class = MODEL_GET_CLASS( filemodel );
 	const char *tname = G_OBJECT_CLASS_NAME( model_class );
@@ -571,12 +618,13 @@ filemodel_load_all( Filemodel *filemodel, Model *parent, const char *filename )
 #endif /*DEBUG*/
 
 	if( model_class->load_text ) {
-		if( !filemodel_load_all_text( filemodel, parent, filename ) ) 
+		if( !filemodel_load_all_text( filemodel, parent, 
+			filename, filename_user ) ) 
 			return( FALSE );
 	}
 	else if( model_class->load ) {
-		if( !filemodel_load_all_xml_file( filemodel, 
-			parent, filename ) )
+		if( !filemodel_load_all_xml_file( filemodel, parent, 
+			filename, filename_user ) )
 			return( FALSE );
 	}
 	else {
@@ -586,8 +634,6 @@ filemodel_load_all( Filemodel *filemodel, Model *parent, const char *filename )
 		return( FALSE );
 	}
 
-	path_add_file( filename );
-
 	/* Don't recomp here, we may be loading a bunch of interdependent
 	 * files.
 	 */
@@ -627,8 +673,6 @@ filemodel_load_all_openfile( Filemodel *filemodel, Model *parent,
 		return( FALSE );
 	}
 
-	path_add_file( of->fname );
-
 	/* Don't recomp here, we may be loading a bunch of interdependent
 	 * files.
 	 */
@@ -640,7 +684,7 @@ filemodel_load_all_openfile( Filemodel *filemodel, Model *parent,
  */
 
 static void
-filemodel_inter_save_cb( iWindow *iwnd, 
+filemodel_inter_saveas_sub_cb( iWindow *iwnd, 
 	void *client, iWindowNotifyFn nfn, void *sys )
 {
 	Filesel *filesel = FILESEL( iwnd );
@@ -648,9 +692,8 @@ filemodel_inter_save_cb( iWindow *iwnd,
 	char *filename;
 
 	if( (filename = filesel_get_filename( filesel )) ) {
-		filemodel_set_filename( filemodel, filename );
-
-		if( filemodel_save_all( filemodel, filename ) ) {
+		if( filemodel_top_save( filemodel, filename ) ) {
+			filemodel_set_filename( filemodel, filename );
 			filemodel_set_modified( filemodel, FALSE );
 			nfn( sys, IWINDOW_YES );
 		}
@@ -675,14 +718,14 @@ filemodel_inter_saveas_cb( iWindow *iwnd, void *client,
 	/* Expands to (eg.) "Save Column A2".
 	 */
 	iwindow_set_title( IWINDOW( filesel ), _( "Save %s %s" ),
-		G_OBJECT_TYPE_NAME( filemodel ), 
+		IOBJECT_GET_CLASS_NAME( filemodel ), 
 		NN( IOBJECT( filemodel )->name ) );
 	filesel_set_flags( filesel, FALSE, TRUE );
 	filesel_set_filetype( filesel, 
 		class->filetype, 
 		watch_int_get( main_watchgroup, class->filetype_pref, 0 ) );
 	iwindow_set_parent( IWINDOW( filesel ), GTK_WIDGET( iwnd ) );
-	filesel_set_done( filesel, filemodel_inter_save_cb, filemodel );
+	filesel_set_done( filesel, filemodel_inter_saveas_sub_cb, filemodel );
 	idialog_set_notify( IDIALOG( filesel ), nfn, sys );
 	iwindow_build( IWINDOW( filesel ) );
 	if( filemodel->filename )
@@ -702,7 +745,7 @@ void
 filemodel_inter_save( iWindow *parent, Filemodel *filemodel )
 {
 	if( filemodel->filename ) {
-		if( !filemodel_save_all( filemodel, filemodel->filename ) ) 
+		if( !filemodel_top_save( filemodel, filemodel->filename ) ) 
 			iwindow_alert( GTK_WIDGET( parent ), 
 				GTK_MESSAGE_ERROR );
 		else 
@@ -742,7 +785,7 @@ filemodel_inter_savenempty_cb( iWindow *iwnd, void *client,
 	iWindowNotifyFn nfn, void *sys )
 {
 	Filemodel *filemodel = FILEMODEL( client );
-	const char *tname = G_OBJECT_TYPE_NAME( filemodel );
+	const char *tname = IOBJECT_GET_CLASS_NAME( filemodel );
 
 	if( filemodel->modified ) {
 		if( filemodel->filename )
@@ -751,11 +794,10 @@ filemodel_inter_savenempty_cb( iWindow *iwnd, void *client,
 				filemodel_inter_empty_cb, filemodel, 
 				nfn, sys, 
 				_( "Object has been modified." ),
-				_( "%s \"%s\" has been modified since you "
+				_( "%s has been modified since you "
 				"loaded it from file \"%s\".\n\n"
 				"Do you want to save your changes?" ),
 				tname, 
-				NN( IOBJECT( filemodel )->name ),
 				NN( filemodel->filename ) );
 		else
 			box_savenosave( GTK_WIDGET( iwnd ), 
@@ -763,10 +805,9 @@ filemodel_inter_savenempty_cb( iWindow *iwnd, void *client,
 				filemodel_inter_empty_cb, filemodel, 
 				nfn, sys, 
 				_( "Object has been modified." ),
-				_( "%s \"%s\" has been modified. "
+				_( "%s has been modified. "
 				"Do you want to save your changes?" ),
-				tname, 
-				NN( IOBJECT( filemodel )->name ) );
+				tname );  
 	}
 	else
 		filemodel_inter_empty_cb( NULL, filemodel, nfn, sys );
@@ -788,7 +829,7 @@ filemodel_inter_close_cb( iWindow *iwnd, void *client,
 {
 	Filemodel *filemodel = FILEMODEL( client );
 
-	IDESTROY( filemodel );
+	iwindow_kill( filemodel_get_window_hint( filemodel ) ); 
 
 	nfn( sys, IWINDOW_YES );
 }
@@ -825,8 +866,8 @@ filemodel_inter_load_cb( iWindow *iwnd,
 	if( (filename = filesel_get_filename( filesel )) ) {
 		filemodel_set_filename( filemodel, filename );
 
-		if( filemodel_load_all( filemodel, 
-			MODEL( parent ), filename ) ) 
+		if( filemodel_load_all( filemodel, MODEL( parent ), 
+			filename, NULL ) ) 
 			nfn( sys, IWINDOW_YES );
 		else
 			nfn( sys, IWINDOW_ERROR );
@@ -847,7 +888,7 @@ filemodel_inter_loadas_cb( iWindow *iwnd, void *client,
 	Filesel *filesel = FILESEL( filesel_new() );
 
 	iwindow_set_title( IWINDOW( filesel ), "Load %s",
-		G_OBJECT_TYPE_NAME( filemodel ) );
+		IOBJECT_GET_CLASS_NAME( filemodel ) );
 	filesel_set_flags( filesel, FALSE, TRUE );
 	filesel_set_filetype( filesel, 
 		class->filetype, 
@@ -892,12 +933,31 @@ filemodel_inter_replace( iWindow *parent, Filemodel *filemodel )
 /* Close all registered filemodels.
  */
 
+/* The first registered, modified filemodel the user hasn't said "ok!!! ffs"
+ * to.
+ */
+static Filemodel *
+filemodel_inter_close_get_filemodel( void )
+{
+	GSList *p;
+
+	for( p = filemodel_registered; p; p = p->next ) {
+		Filemodel *filemodel = FILEMODEL( p->data );
+
+		if( filemodel->modified )
+			return( filemodel );
+	}
+
+	return( NULL );
+}
+
 void
 filemodel_inter_close_registered_cb( iWindow *iwnd, void *client, 
 	iWindowNotifyFn nfn, void *sys )
 {
-	if( filemodel_registered ) {
-		Filemodel *filemodel = FILEMODEL( filemodel_registered->data );
+	Filemodel *filemodel; 
+
+	if( (filemodel = filemodel_inter_close_get_filemodel()) ) { 
 		iWindowSusp *susp = iwindow_susp_new( 
 			filemodel_inter_close_registered_cb, 
 			iwnd, client, nfn, sys );
