@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -40,6 +40,10 @@
 #define DEBUG_GETMEM
  */
 
+/* Time each GC, handy for benchmarking.
+#define DEBUG_GC_TIME
+ */
+
 #include "ip.h"
 
 static iObjectClass *parent_class = NULL;
@@ -631,7 +635,7 @@ heap_gc( Heap *heap )
 	int ncells;
 	int nblocks;
 
-#ifdef DEBUG
+#ifdef DEBUG_GC_TIME
 	static GTimer *GC_timer = NULL;
 
 	if( !GC_timer )
@@ -640,7 +644,7 @@ heap_gc( Heap *heap )
 	g_timer_reset( GC_timer );
 
 	printf( "heap_gc: starting GC for heap %s\n", IOBJECT( heap )->name );
-#endif /*DEBUG*/
+#endif /*DEBUG_GC_TIME*/
 
 	/* Clear marks on managed objects. Nodes should all be clear already.
 	 */
@@ -690,11 +694,11 @@ heap_gc( Heap *heap )
 	 */
 	managed_free_unused( heap );
 
-#ifdef DEBUG
+#ifdef DEBUG_GC_TIME
 	printf( "heap_gc: %d cells in %d blocks, %d in use\n",
 		ncells, nblocks, ncells - nfree );
 	printf( "(GC took %gs)\n",  g_timer_elapsed( GC_timer, NULL ) );
-#endif 
+#endif /*DEBUG_GC_TIME*/
 
 	return( TRUE );
 }
@@ -1248,7 +1252,7 @@ heap_map_list( PElement *base, heap_map_list_fn fn, void *a, void *b )
 		/* Apply user function to the head.
 		 */
 		PEGETHD( &head, &e );
-		if(( res = fn( &head, a, b )) )
+		if( (res = fn( &head, a, b )) )
 			return( res );
 
 		/* Reduce the tail.
@@ -1761,12 +1765,30 @@ heap_copy( Heap *heap, Compile *compile, PElement *out )
 	Element *root = &compile->base;
 	HeapNode *ri[MAX_RELOC];
 
+	/* Check for possible C stack overflow ... can't go over 2M on most
+	 * systems if we're using (or any of our libs are using) threads.
+	 */
+	if( (char *) main_c_stack_base - (char *) &heap > 2000000 ) {
+		error_top( _( "Overflow error." ) );
+		error_sub( _( "C stack overflow. Circular definition." ) );
+		return( FALSE ); 
+	}
+
 #ifdef DEBUG
 	printf( "heap_copy: " );
 	symbol_name_print( compile->sym );
 	printf( "\n" );
 #endif /*DEBUG*/
 
+	/* Check for possible C stack overflow ... can't go over 2M on most
+	 * systems if we're using (or any of our libs are using) threads.
+	 */
+	if( (char *) main_c_stack_base - (char *) &heap > 2000000 ) {
+		error_top( _( "Overflow error." ) );
+		error_sub( _( "C stack overflow. Expression too complex." ) );
+			return( FALSE );
+	}
+
 	switch( root->type ) {
 	case ELEMENT_NODE:
 		/* Need a tree copy.
@@ -1883,7 +1905,9 @@ heap_gvalue_to_ip( GValue *in, PElement *out )
 		PEPUTP( out, ELEMENT_BOOL, (int) g_value_get_boolean( in ) );
 	}
 	else if( G_VALUE_HOLDS_CHAR( in ) ) {
-		PEPUTP( out, ELEMENT_CHAR, (int) g_value_get_char( in ) );
+		/* g_value_get_schar() is not in older glibs.
+		 */
+		PEPUTP( out, ELEMENT_CHAR, (int) g_value_get_uchar( in ) );
 	}
 	else if( G_VALUE_HOLDS_UCHAR( in ) ) {
 		PEPUTP( out, ELEMENT_CHAR, (int) g_value_get_uchar( in ) );
@@ -2070,7 +2094,7 @@ lisp_list( VipsBuf *buf, PElement *base,
 /* Print a [char] ... fall back to lisp_list() if we hit a non-char
  * element. base is the RHS of a cons, so it can be a managedstring too.
  */
-static void
+static gboolean
 lisp_string( VipsBuf *buf, PElement *base, 
 	GSList **back, gboolean fn, int indent )
 {
@@ -2089,7 +2113,8 @@ lisp_string( VipsBuf *buf, PElement *base,
 				vips_buf_appendf( buf, "%c", PEGETCHAR( &pe ) );
 
 				PEPOINTRIGHT( hn, &pe );
-				lisp_string( buf, &pe, back, fn, indent );
+				(void) lisp_string( buf, 
+					&pe, back, fn, indent );
 			}
 			else {
 				vips_buf_appends( buf, "\":[" );
@@ -2109,6 +2134,8 @@ lisp_string( VipsBuf *buf, PElement *base,
 		vips_buf_appends( buf, PEGETMANAGEDSTRING( base )->string );
 	else if( !PEISELIST( base ) ) 
 		error = TRUE;
+
+	return( error ); 
 }
 
 /* Print a graph LISP-style.
@@ -2168,7 +2195,7 @@ lisp_node( VipsBuf *buf, HeapNode *hn, GSList **back, gboolean fn, int indent )
 		if( PEISCHAR( &p1 ) ) {
 			vips_buf_appendf( buf, "\"%c", PEGETCHAR( &p1 ) );
 			PEPOINTRIGHT( hn, &p2 );
-			lisp_string( buf, &p2, back, fn, indent );
+			(void) lisp_string( buf, &p2, back, fn, indent );
 			vips_buf_appends( buf, "\"" );
 		}
 		else {
