@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -299,7 +299,7 @@ typedef int (*heap_max_fn)( Heap * );
 struct _Heap {
 	iObject parent_object;
 
-	Compile *compile;	/* Debugging ... if non-null, assos. compile */
+	Compile *compile;	/* If non-null, assoc. compile */
 
 	heap_max_fn max_fn;	/* Max nodes in this heap */
 	int mxb;		/* Max blocks until next check */
