@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -169,9 +169,9 @@ iimage_save( Model *model, xmlNode *xnode )
 
 	/* We always rebuild the value from the expr ... don't save.
 	 */
-	if( !set_prop( xthis, "image_left", "%d", iimage->image_left ) ||
-		!set_prop( xthis, "image_top", "%d", iimage->image_top ) ||
-		!set_prop( xthis, "image_mag", "%d", iimage->image_mag ) ||
+	if( !set_iprop( xthis, "image_left", iimage->image_left ) ||
+		!set_iprop( xthis, "image_top", iimage->image_top ) ||
+		!set_iprop( xthis, "image_mag", iimage->image_mag ) ||
 		!set_sprop( xthis, "show_status",
 			bool_to_char( iimage->show_status ) ) ||
 		!set_sprop( xthis, "show_paintbox",
@@ -333,13 +333,20 @@ iimage_graphic_save( Classmodel *classmodel,
 {
 	iImage *iimage = IIMAGE( classmodel );
 	ImageValue *value = &iimage->value;
-	char buf[MAX_STRSIZE];
+	char buf[FILENAME_MAX];
 
 	/* Can't happen nested-ly, so a static is OK. 
 	 */
 	static GTimer *timer = NULL;
 
-	strcpy( buf, filename );
+	/* We don't want $VAR etc. in the filename we pass down to the file
+	 * ops.
+	 */
+	im_strncpy( buf, filename, FILENAME_MAX );
+	path_expand( buf );
+
+	/* Append the mode string. This needs an expanded filename.
+	 */
 	filesel_add_mode( buf );
 
 	if( !timer )
@@ -379,7 +386,7 @@ iimage_replace( iImage *iimage, const char *filename )
 
 	if( itext_set_formula( itext, vips_buf_all( &buf ) ) ) {
 		itext_set_edited( itext, TRUE );
-		filemodel_set_modified( FILEMODEL( row->ws ), TRUE );
+		workspace_set_modified( row->ws, TRUE );
 		(void) expr_dirty( row->expr, link_serial_new() );
 
 		mainw_recent_add( &mainw_recent_image, filename );
@@ -414,6 +421,7 @@ iimage_class_init( iImageClass *class )
 	gobject_class->dispose = iimage_dispose;
 	gobject_class->finalize = iimage_finalize;
 
+	iobject_class->user_name = _( "Image" );
 	iobject_class->generate_caption = iimage_generate_caption;
 	iobject_class->info = iimage_info;
 
