@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -85,8 +85,8 @@ most of the jobs above are pushed down into vips8 now ... except for
 #define DEBUG
 #define DEBUG_MAKE
 #define DEBUG_RGB
-#define DEBUG_CHECK
 #define DEBUG_OPEN
+#define DEBUG_CHECK
  */
 
 static iContainerClass *imageinfogroup_parent_class = NULL;
@@ -709,7 +709,7 @@ imageinfo_proxy_add( Imageinfo *imageinfo )
 {
 	/* Only if we're running interactively.
 	 */
-	if( !main_window_top )
+	if( main_option_batch )
 		return;
 
 	/* Already being monitored?
@@ -901,13 +901,15 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 gboolean 
 imageinfo_is_from_file( Imageinfo *imageinfo )
 {
-	return( IOBJECT( imageinfo )->name && imageinfo->from_file );
+	return( IOBJECT( imageinfo )->name && 
+		imageinfo->from_file );
 }
 
 static gint
 imageinfo_attach_check_cb( Imageinfo *imageinfo )
 {
-	if( imageinfo_is_from_file( imageinfo ) && imageinfo->check_tid ) {
+	if( imageinfo_is_from_file( imageinfo ) && 
+		imageinfo->check_tid ) {
 		struct stat buf;
 
 		if( !stat( IOBJECT( imageinfo )->name, &buf ) &&
@@ -925,10 +927,11 @@ imageinfo_attach_check_cb( Imageinfo *imageinfo )
 static void
 imageinfo_attach_check( Imageinfo *imageinfo )
 {
-	if( imageinfo_is_from_file( imageinfo ) && !imageinfo->check_tid ) {
+	if( imageinfo_is_from_file( imageinfo ) && 
+		!imageinfo->check_tid ) {
 		struct stat buf;
 
-		/* Need to be able to state to be able to track a file.
+		/* Need to be able to stat() to be able to track a file.
 		 */
 		if( stat( IOBJECT( imageinfo )->name, &buf ) )
 			return;
@@ -1154,8 +1157,10 @@ imageinfo_check_paintable( Imageinfo *imageinfo, GtkWidget *parent,
 {
 	IMAGE *im = imageinfo_get( FALSE, imageinfo );
 
-	if( im_isfile( im ) && 
-		!imageinfo->dfile && !imageinfo->ok_to_paint ) {
+	if( im &&
+		im_isfile( im ) && 
+		!imageinfo->dfile && 
+		!imageinfo->ok_to_paint ) {
 		iDialog *idlg;
 
 		idlg = box_yesno( parent,
@@ -1175,7 +1180,9 @@ imageinfo_check_paintable( Imageinfo *imageinfo, GtkWidget *parent,
 
 		return( FALSE );
 	}
-	else if( !im_isfile( im ) && !imageinfo->ok_to_paint ) {
+	else if( im &&
+		!im_isfile( im ) && 
+		!imageinfo->ok_to_paint ) {
 		if( !imageinfo_make_paintable( imageinfo ) ) {
 			nfn( sys, IWINDOW_ERROR );
 			return( FALSE );
