@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -233,6 +233,10 @@ imageview_get_region_type( GtkAction *action )
 			return( (iRegionType) i );
 
 	g_assert( FALSE );
+	
+	/* Keep gcc happy.
+	 */
+	return( FALSE );
 }
 
 static void
@@ -314,7 +318,7 @@ imageview_new_arrow4_action_cb( GtkAction *action, Imageview *iv )
 	}
 
 	col = sym->expr->row->top_col;
-	model_scrollto( MODEL( col ), MODEL_SCROLL_BOTTOM );
+	column_scrollto( col, MODEL_SCROLL_BOTTOM );
 }
 
 static void
@@ -448,27 +452,27 @@ static GtkActionEntry imageview_actions[] = {
 	/* Actions.
 	 */
 	{ "NewMark", 
-		GTK_STOCK_NEW, N_( "New _Mark" ), NULL, 
+		NULL, N_( "_Mark" ), NULL, 
 		N_( "Create a new mark" ), 
 		G_CALLBACK( imageview_new_arrow2_action_cb ) },
 
 	{ "NewHGuide", 
-		GTK_STOCK_NEW, N_( "New _Horizontal Guide" ), NULL, 
+		NULL, N_( "_Horizontal Guide" ), NULL, 
 		N_( "Create a new horizontal guide" ), 
 		G_CALLBACK( imageview_new_arrow2_action_cb ) },
 
 	{ "NewVGuide", 
-		GTK_STOCK_NEW, N_( "New _Vertical Guide" ), NULL, 
+		NULL, N_( "_Vertical Guide" ), NULL, 
 		N_( "Create a new vertical guide" ), 
 		G_CALLBACK( imageview_new_arrow2_action_cb ) },
 
 	{ "NewArrow", 
-		GTK_STOCK_NEW, N_( "New _Arrow" ), NULL, 
+		NULL, N_( "_Arrow" ), NULL, 
 		N_( "Create a new arrow" ), 
 		G_CALLBACK( imageview_new_arrow4_action_cb ) },
 
 	{ "NewRegion", 
-		GTK_STOCK_NEW, N_( "New _Region" ), NULL, 
+		NULL, N_( "_Region" ), NULL, 
 		N_( "Create a new region" ), 
 		G_CALLBACK( imageview_new_arrow4_action_cb ) },
 
