@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -49,8 +49,8 @@
 /* But some themes can trigger warnings, argh, so sometimes we need to
  * undef it. VipsObject sets can trigger warnings. libgoffice will warn about
  * precision issues if run under valgrind.
-#undef DEBUG_FATAL
  */
+#undef DEBUG_FATAL
 
 /* Time startup.
 #define DEBUG_TIME
@@ -67,10 +67,7 @@
 
 /* General stuff. 
  */
-GdkWindow *main_window_gdk;			/* Top GdkWindow */
-GtkWidget *main_window_top = NULL;		/* Secret top window */
-
-Workspacegroup *main_workspacegroup = NULL;	/* All the workspaces */
+Workspaceroot *main_workspaceroot = NULL;	/* All the workspaces */
 Toolkitgroup *main_toolkitgroup = NULL;		/* All the toolkits */
 Symbol *main_symbol_root = NULL;		/* Root of symtable */
 Watchgroup *main_watchgroup = NULL;		/* All of the watches */
@@ -95,6 +92,7 @@ static char *main_option_output = NULL;
 static char **main_option_set = NULL;
 static gboolean main_option_benchmark = FALSE;
 gboolean main_option_time_save = FALSE;
+gboolean main_option_profile = FALSE;
 gboolean main_option_i18n = FALSE;
 static gboolean main_option_print_main = FALSE;
 static gboolean main_option_version = FALSE;
@@ -135,6 +133,9 @@ static GOptionEntry main_option[] = {
 	{ "time-save", 't', 0, G_OPTION_ARG_NONE, &main_option_time_save, 
 		N_( "time image save operations" ), 
 		NULL },
+	{ "profile", 'r', 0, G_OPTION_ARG_NONE, &main_option_profile, 
+		N_( "profile workspace calculation" ), 
+		NULL },
 	{ "prefix", 'x', 0, G_OPTION_ARG_FILENAME, &main_option_prefix, 
 		N_( "start as if installed to PREFIX" ), "PREFIX" },
 	{ "i18n", 'i', 0, G_OPTION_ARG_NONE, &main_option_i18n, 
@@ -197,9 +198,13 @@ main_error_exit( const char *fmt, ... )
         va_start( args, fmt );
         (void) vfprintf( stderr, fmt, args );
         va_end( args );
+	fprintf( stderr, "\n" );  
 
-	fprintf( stderr, "\n%s %s\n", 
-		error_get_top(), error_get_sub() );
+	if( strcmp( error_get_top(), "" ) != 0 ) {
+		fprintf( stderr, "%s\n", error_get_top() ); 
+		if( strcmp( error_get_sub(), "" ) != 0 ) 
+			fprintf( stderr, "%s\n", error_get_sub() ); 
+	}
 
 	if( main_option_verbose ) {
 		char txt[MAX_STRSIZE];
@@ -207,7 +212,7 @@ main_error_exit( const char *fmt, ... )
 
 		slist_map( expr_error_all,
 			(SListMapFn) expr_error_print, &buf );
-		fprintf( stderr, "%s\n", vips_buf_all( &buf ) );
+		fprintf( stderr, "%s", vips_buf_all( &buf ) );
 	}
 
 	exit( 1 );
@@ -221,7 +226,7 @@ main_print_main( Symbol *sym )
 	PElement *root;
 
 	root = &sym->expr->root;
-	if( symbol_recalculate_check( sym ) ||
+	if( !symbol_recalculate_check( sym ) ||
 		!reduce_pelement( reduce_context, reduce_spine_strict, root ) ) 
 		main_error_exit( _( "error calculating \"%s\"" ), 
 			symbol_name_scope( sym ) );
@@ -244,8 +249,7 @@ main_print_ws( Workspace *ws, gboolean *found )
 {
 	Symbol *sym;
 
-	if( (sym = compile_lookup( 
-		ws->sym->expr->compile, "main" )) ) {
+	if( (sym = compile_lookup( ws->sym->expr->compile, "main" )) ) {
 		main_print_main( sym );
 		*found = TRUE;
 	}
@@ -267,7 +271,8 @@ main_quit( void )
 	printf( "main_quit: cleaning up ...\n" );
 #endif/*DEBUG*/
 
-	if( main_option_print_main || main_option_output ) {
+	if( main_option_print_main || 
+		main_option_output ) {
 		Symbol *sym;
 		gboolean found;
 
@@ -308,7 +313,7 @@ main_quit( void )
 
 	/* Remove any ws retain files.
 	 */
-	workspace_retain_clean();
+	workspacegroup_autosave_clean();
 
 	/* Junk all symbols. This may remove a bunch of intermediate images
 	 * too.
@@ -316,7 +321,7 @@ main_quit( void )
 	UNREF( main_watchgroup );
 	UNREF( main_symbol_root );
 	UNREF( main_toolkitgroup );
-	UNREF( main_workspacegroup );
+	UNREF( main_workspaceroot );
 
 	/* Junk reduction machine ... this should remove all image temps.
 	 */
@@ -336,6 +341,8 @@ main_quit( void )
 		libgoffice_shutdown ();
 #endif /*HAVE_LIBGOFFICE*/
 
+	path_rewrite_free_all();
+
 	/* Should have freed everything now.
 	 */
 
@@ -406,7 +413,7 @@ main_quit_test( void )
 
 	/* Close registered models.
 	 */
-	filemodel_inter_close_registered_cb( IWINDOW( main_window_top ), NULL,
+	filemodel_inter_close_registered_cb( iwindow_pick_one(), NULL,
 		main_quit_test_cb, NULL );
 }
 
@@ -423,23 +430,25 @@ main_watchgroup_changed_cb( void )
 /* Try to load a thing, anything at all. Actually, we don't load plugins
  * experimentally, win32 pops up an annoying error dialog if you try that.
  */
-gboolean
+static gboolean
 main_load( Workspace *ws, const char *filename )
 {
-	Workspace *new_ws;
+	Workspacegroup *new_wsg;
 
-	if( (new_ws = 
-		workspace_new_from_file( main_workspacegroup, filename )) ) {
-		Mainw *new_mainw;
+	if( (new_wsg = workspacegroup_new_from_file( main_workspaceroot, 
+		filename, filename )) ) {
+		Mainw *mainw;
 
 		if( !main_option_batch ) {
-			new_mainw = mainw_new( new_ws );
-			gtk_widget_show( GTK_WIDGET( new_mainw ) );
+			mainw = mainw_new( new_wsg );
+			gtk_widget_show( GTK_WIDGET( mainw ) );
 		}
+
 		mainw_recent_add( &mainw_recent_workspace, filename );
 
 		return( TRUE );
 	}
+
 	error_clear();
 
 	/* workspace_load_file() needs to recalc to work, try to avoid that by
@@ -498,20 +507,21 @@ main_load_def( const char *filename )
 }
 
 static void *
-main_load_ws( const char *filename )
+main_load_wsg( const char *filename )
 {
-	Workspace *ws;
+	Workspacegroup *wsg;
 
 #ifdef DEBUG
-	printf( "main_load_ws: %s\n", filename );
+	printf( "main_load_wsg: %s\n", filename );
 #endif/*DEBUG*/
 
 	progress_update_loading( 0, im_skip_dir( filename ) );
 
-	if( !(ws = workspace_new_from_file( main_workspacegroup, filename )) ) 
+	if( !(wsg = workspacegroup_new_from_file( main_workspaceroot, 
+		filename, filename )) ) 
 		iwindow_alert( NULL, GTK_MESSAGE_ERROR );
 	else {
-		filemodel_set_auto_load( FILEMODEL( ws ) );
+		filemodel_set_auto_load( FILEMODEL( wsg ) );
 	}
 
 	return( NULL );
@@ -580,7 +590,7 @@ main_load_startup( void )
 
 	/* Load any plug-ins on PATH_START. 
 	 */
-	(void) path_map_exact( PATH_START, "*.plg", 
+	(void) path_map( PATH_START, "*.plg", 
 		(path_map_fn) main_load_plug, NULL );
 
 	/* Link all VIPS functions as SYM_EXTERNAL.
@@ -593,14 +603,14 @@ main_load_startup( void )
 #ifdef DEBUG
 	printf( "definitions init\n" );
 #endif/*DEBUG*/
-	(void) path_map_exact( PATH_START, "*.def", 
+	(void) path_map( PATH_START, "*.def", 
 		(path_map_fn) main_load_def, NULL );
 
 #ifdef DEBUG
 	printf( "ws init\n" );
 #endif/*DEBUG*/
-	(void) path_map_exact( PATH_START, "*.ws", 
-		(path_map_fn) main_load_ws, NULL );
+	(void) path_map( PATH_START, "*.ws", 
+		(path_map_fn) main_load_wsg, NULL );
 
 	mainw_recent_thaw();
 }
@@ -651,12 +661,10 @@ main_file_for_stock( GtkIconFactory *icon_factory,
 	GtkIconSource *icon_source;
 	GtkIconSet *icon_set;
 	char buf[FILENAME_MAX];
-	char buf2[FILENAME_MAX];
 
-	im_snprintf( buf2, FILENAME_MAX, 
+	im_snprintf( buf, FILENAME_MAX, 
 		"$VIPSHOME/share/$PACKAGE/data/%s", file );
-	expand_variables( buf2, buf );
-        nativeize_path( buf );
+	path_expand( buf );
 	icon_source = gtk_icon_source_new(); 
 	gtk_icon_source_set_filename( icon_source, buf );
 	icon_set = gtk_icon_set_new();
@@ -690,6 +698,7 @@ main_register_icons( void )
 		{ STOCK_RECT, N_( "Fill Rectangle" ), 0, 0, GETTEXT_PACKAGE },
 		{ STOCK_MOVE, N_( "Pan" ), 0, 0, GETTEXT_PACKAGE },
 		{ STOCK_SELECT, N_( "Select" ), 0, 0, GETTEXT_PACKAGE },
+		{ STOCK_LOCK, N_( "Locked" ), 0, 0, GETTEXT_PACKAGE },
 
 		/* And the LEDs we use.
 		 */
@@ -743,6 +752,10 @@ main_register_icons( void )
 		STOCK_MOVE, "stock-tool-move-22.png" );
 	main_file_for_stock( main_icon_factory, 
 		STOCK_SELECT, "stock-tool-select-22.png" );
+	main_file_for_stock( main_icon_factory, 
+		STOCK_LOCK, "stock-padlock-closed-22.png" );
+	main_file_for_stock( main_icon_factory, 
+		STOCK_ALERT, "stock-alert-22.png" );
 	main_file_for_stock( main_icon_factory, 
 		STOCK_LED_RED, "stock-led-red-18.png" );
 	main_file_for_stock( main_icon_factory, 
@@ -767,9 +780,6 @@ main_x_init( int *argc, char ***argv )
 {
 	char buf[FILENAME_MAX];
 
-	if( main_window_gdk )
-		return;
-
 #ifdef DEBUG
 	printf( "X11 init\n" );
 #endif/*DEBUG*/
@@ -789,8 +799,9 @@ main_x_init( int *argc, char ***argv )
 
 	/* Set the default icon. 
 	 */
-	expand_variables( "$VIPSHOME/share/$PACKAGE/data/vips-128.png", buf );
-        nativeize_path( buf );
+	im_strncpy( buf, 
+		"$VIPSHOME/share/$PACKAGE/data/vips-128.png", FILENAME_MAX );
+	path_expand( buf );
 	gtk_window_set_default_icon_from_file( buf, NULL );
 
 	/* Turn off startup notification. Startup is done when we pop our
@@ -798,13 +809,6 @@ main_x_init( int *argc, char ***argv )
 	 */
 	gtk_window_set_auto_startup_notification( FALSE );
 
-        /* Make invisible top level window ... used to get stuff for
-         * build. Realize it, but don't display it.
-         */
-        main_window_top = iwindow_new( GTK_WINDOW_TOPLEVEL );
-        gtk_widget_realize( main_window_top );
-	main_window_gdk = main_window_top->window;
-
 #ifdef DEBUG_UPDATES
 	printf( "*** debug updates is on\n" );
 	gdk_window_set_debug_updates( TRUE );
@@ -869,9 +873,8 @@ main_check_temp( double total )
 		VipsBuf buf = VIPS_BUF_STATIC( txt );
 		char tmp[FILENAME_MAX];
 
-		expand_variables( PATH_TMP, tmp );
-		nativeize_path( tmp );
-		absoluteize_path( tmp );
+		im_strncpy( tmp, PATH_TMP, FILENAME_MAX );
+		path_expand( tmp );
 		vips_buf_append_size( &buf, total );
 
 		box_yesno( NULL,
@@ -949,12 +952,49 @@ main_set( const char *str )
 	return( TRUE );
 }
 
+static char prefix_buffer[FILENAME_MAX];
+static gboolean prefix_valid = FALSE;
+
+/* Override the install guess from vips. Handy for testing. 
+ */
+static void
+set_prefix( const char *prefix )
+{
+	im_strncpy( prefix_buffer, prefix, FILENAME_MAX );
+	nativeize_path( prefix_buffer );
+	absoluteize_path( prefix_buffer );
+	setenvf( "VIPSHOME", "%s", prefix_buffer );
+	prefix_valid = TRUE;
+}
+
+/* Guess VIPSHOME, if we can.
+ */
+const char *
+get_prefix( void )
+{
+	if( !prefix_valid ) {
+		const char *prefix;
+
+		if( !(prefix = im_guess_prefix( main_argv0, "VIPSHOME" )) ) {
+			error_top( _( "Unable to find install area." ) );
+			error_vips();
+
+			return( NULL );
+		}
+
+		set_prefix( prefix ); 
+	}
+
+	return( prefix_buffer );
+}
+
 /* Start here!
  */
 int
 main( int argc, char *argv[] )
 {
 	gboolean welcome_message = FALSE;
+	Workspacegroup *wsg;
 	Workspace *ws;
 	GError *error = NULL;
 	GOptionContext *context;
@@ -1061,7 +1101,7 @@ main( int argc, char *argv[] )
 	/* Init i18n ... get catalogues from $VIPSHOME/share/locale so we're
 	 * relocatable.
 	 */
-	prefix = im_guess_prefix( main_argv0, "VIPSHOME" );
+	prefix = get_prefix();
 	im_snprintf( name, 256, 
 		"%s" G_DIR_SEPARATOR_S "share" G_DIR_SEPARATOR_S "locale", 
 		prefix );
@@ -1094,15 +1134,8 @@ main( int argc, char *argv[] )
 	 * cats sadly :( since we have to init i18n before arg parsing. Handy
 	 * for testing without installing.
 	 */
-	if( main_option_prefix ) {
-		char tmp[FILENAME_MAX];
-
-		im_strncpy( tmp, main_option_prefix, FILENAME_MAX );
-		nativeize_path( tmp );
-		absoluteize_path( tmp );
-		prefix = im_strdupn( tmp );
-		setenvf( "VIPSHOME", "%s", prefix );
-	}
+	if( main_option_prefix ) 
+		set_prefix( main_option_prefix );
 
 	if( main_option_version ) {
 		printf( "%s-%s", PACKAGE, VERSION );
@@ -1209,10 +1242,10 @@ main( int argc, char *argv[] )
 	g_object_ref( G_OBJECT( main_symbol_root ) );
 	iobject_sink( IOBJECT( main_symbol_root ) );
 	model_base_init();
-	main_workspacegroup = workspacegroup_new( "Workspaces" );
-	g_object_ref( G_OBJECT( main_workspacegroup ) );
-	iobject_sink( IOBJECT( main_workspacegroup ) );
-	main_watchgroup = watchgroup_new( main_workspacegroup, "Preferences" );
+	main_workspaceroot = workspaceroot_new( "Workspaces" );
+	g_object_ref( G_OBJECT( main_workspaceroot ) );
+	iobject_sink( IOBJECT( main_workspaceroot ) );
+	main_watchgroup = watchgroup_new( main_workspaceroot, "Preferences" );
 	g_object_ref( G_OBJECT( main_watchgroup ) );
 	iobject_sink( IOBJECT( main_watchgroup ) );
 	main_toolkitgroup = toolkitgroup_new( symbol_root );
@@ -1274,6 +1307,14 @@ main( int argc, char *argv[] )
 	 */
 	main_load_startup();
 
+	/* Recalc to build all classes and gets prefs working. 
+	 *
+	 * We have to do this in batch
+	 * mode since we can find dirties through dynamic lookups. Even though
+	 * you might think we could just follow recomps.
+	 */
+	symbol_recalculate_all_force( TRUE );
+
 #ifdef DEBUG
 	printf( "arg processing\n" );
 #endif/*DEBUG*/
@@ -1281,6 +1322,7 @@ main( int argc, char *argv[] )
 	/* Might make this from stdin/whatever if we have a special
 	 * command-line flag.
 	 */
+	wsg = NULL;
 	ws = NULL;
 
 	/* Second command-line pass. This time we do any actions.
@@ -1308,20 +1350,22 @@ main( int argc, char *argv[] )
 	}
 
 	if( main_option_stdin_ws ) {
-		if( !(ws = workspace_new_from_openfile( 
-			main_workspacegroup, main_stdin )) ) 
+		if( !(wsg = workspacegroup_new_from_openfile( 
+			main_workspaceroot, main_stdin )) ) 
 			main_log_add( "%s\n", error_get_sub() );
 		else 
 			/* Don't want to have "stdin" as the filename.
 			 */
-			filemodel_set_filename( FILEMODEL( ws ), NULL );
+			filemodel_set_filename( FILEMODEL( wsg ), NULL );
 	}
 
-	/* Make sure we have a start workspace.
+	/* Make a start workspace and workspacegroup to load
+	 * stuff into.
 	 */
-	if( !ws ) {
-		workspacegroup_name_new( main_workspacegroup, name );
-		ws = workspace_new_blank( main_workspacegroup, name );
+	if( !wsg ) {
+		wsg = workspacegroup_new_blank( main_workspaceroot, NULL ); 
+		ws = WORKSPACE( icontainer_get_nth_child( 
+			ICONTAINER( wsg ), 0 ) ); 
 	}
 
 	/* Reset IM_CONCURRENCY if a watch changes. Need to do this after
@@ -1330,12 +1374,6 @@ main( int argc, char *argv[] )
 	g_signal_connect( main_watchgroup, "watch_changed", 
 		G_CALLBACK( main_watchgroup_changed_cb ), NULL );
 
-	/* Recalc to build all classes. We have to do this in batch
-	 * mode since we can find dirties through dynamic lookups. Even though
-	 * you might think we could just follow recomps.
-	 */
-	symbol_recalculate_all_force( TRUE );
-
 	/* Pass PATH_TMP down to vips via TMPDIR. See im_system(), for
 	 * example. We need to do this after the first recomp so that prefs
 	 * are loaded.
@@ -1343,9 +1381,11 @@ main( int argc, char *argv[] )
 {
 	char buf[FILENAME_MAX];
 
-	expand_variables( PATH_TMP, buf );
-	nativeize_path( buf );
+	im_strncpy( buf, PATH_TMP, FILENAME_MAX );
+	path_expand( buf );
 	setenvf( "TMPDIR", "%s", buf );
+
+	path_rewrite_add( PATH_TMP, "$TMPDIR", TRUE );
 }
 
 	/* Measure amount of stuff in temp area ... need this for checking
@@ -1366,6 +1406,15 @@ main( int argc, char *argv[] )
 	vips_buf_rewind( &buf );
 	vips_buf_appendf( &buf, "argv = [" );
 	for( i = 0; i < argc; i++ ) {
+		/* Ignore "--" args. Consider eg. 
+		 *
+		 * 	./try201.nip2 -o x.v -- -12 ~/pics/shark.jpg 
+		 *
+		 * if we didn't remove --, all scripts would need to. 
+		 */
+		if( strcmp( argv[i], "--" ) == 0 )
+			continue;
+
 		if( i > 0 )
 			vips_buf_appendf( &buf, ", " );
 		vips_buf_appendf( &buf, "\"%s\"", argv[i] );
@@ -1378,15 +1427,45 @@ main( int argc, char *argv[] )
 
 	filemodel_set_modified( FILEMODEL( kit ), FALSE );
 
+	/* Double-check: we often forget to move the prefs ws to the latest
+	 * version.
+	 */
+#ifdef DEBUG_LEAK
+{
+	Symbol *wsr_sym = main_workspaceroot->sym;
+	Symbol *ws_sym = SYMBOL( icontainer_child_lookup( 
+		ICONTAINER( wsr_sym->expr->compile ), "Preferences" ) );
+
+	if( !ws_sym )
+		printf( "No prefs workspace!\n" ); 
+	else {
+		Workspace *ws = ws_sym->ws;
+
+		if( ws->compat_major || 
+			ws->compat_minor )
+			printf( "Preferences loaded in compat mode!\n" );
+	}
+}
+#endif /*DEBUG_LEAK*/
+
 	if( !main_option_no_load_args ) {
 		/* Load args as files, if we can. 
 		 */
-		for( i = 1; i < argc; i++ )
-			if( !main_load( ws, argv[i] ) ) 
+		for( i = 1; i < argc; i++ ) {
+			char buf[FILENAME_MAX];
+
+			/* We want to use the absolute, compact form of the 
+			 * filename object so we don't get a dependency on CWD.
+			 */
+			im_strncpy( buf, argv[i], FILENAME_MAX );
+			path_compact( buf );
+
+			if( !main_load( ws, buf ) ) 
 				main_log_add( "%s\n", error_get_sub() );
+		}
 	}
 
-	/* Abandon batch mode if there are startup errors.
+	/* In batch mode give up if there are startup errors.
 	 */
 	if( main_option_batch ) {
 		if( !main_log_is_empty() ) {
@@ -1407,7 +1486,17 @@ main( int argc, char *argv[] )
 	/* Make sure our start ws doesn't have modified set. We may have
 	 * loaded some images or whatever into it.
 	 */
-	filemodel_set_modified( FILEMODEL( ws ), FALSE );
+	workspace_set_modified( ws, FALSE );
+
+	/* If the start ws is empty (we didn't load anything into it) and we
+	 * loaded some other workspaces, we can junk the empty ws. 
+	 */
+	if( icontainer_get_n_children( ICONTAINER( main_workspaceroot ) ) > 2 &&
+		workspace_is_empty( ws ) ) {
+		IDESTROY( wsg ); 
+		wsg = NULL;
+		ws = NULL;
+	}
 
 #ifdef DEBUG_TIME
 	printf( "DEBUG_TIME: main init in %gs\n",  
@@ -1417,13 +1506,10 @@ main( int argc, char *argv[] )
 	/* Are we running interactively? Start the main window and loop.
 	 */
 	if( !main_option_batch ) {
-		/* Only display our initial ws if it's not blank, or if it is
-		 * blank, if there are no other windows up.
-		 */
-		if( !workspace_is_empty( ws ) || mainw_number() == 0 ) {
+		if( wsg ) { 
 			Mainw *mainw;
-			
-			mainw = mainw_new( ws );
+
+			mainw = mainw_new( wsg );
 			gtk_widget_show( GTK_WIDGET( mainw ) );
 		}
 
@@ -1447,8 +1533,8 @@ main( int argc, char *argv[] )
 
 			im_snprintf( buf, 256, 
 				_( "Welcome to %s-%s!" ), PACKAGE, VERSION );
-			expand_variables( get_savedir(), save_dir );
-			nativeize_path( save_dir );
+			im_strncpy( save_dir, get_savedir(), FILENAME_MAX );
+			path_expand( save_dir );
 			error_top( "%s", buf );
 			error_sub( 
 _( "A new directory has been created to hold startup, "
@@ -1460,20 +1546,14 @@ _( "A new directory has been created to hold startup, "
 			iwindow_alert( NULL, GTK_MESSAGE_INFO );
 		}
 
-		/* Offer to junk temps.
-		 */
 		main_check_temp( total );
 
 #ifdef DEBUG
 		printf( "starting event dispatch loop\n" );
 #endif/*DEBUG*/
 
-		/* Through startup.
-		 */
 		main_starting = FALSE;
 
-		/* Make sure we have a recalc queued.
-		 */
 		symbol_recalculate_all_force( FALSE );
 
 		gtk_main();
