@@ -51,10 +51,10 @@
 /* VOID:OBJECT,INT (nipmarshal.list:25) */
 void
 nip_VOID__OBJECT_INT (GClosure     *closure,
-                      GValue       *return_value,
+                      GValue       *return_value G_GNUC_UNUSED,
                       guint         n_param_values,
                       const GValue *param_values,
-                      gpointer      invocation_hint,
+                      gpointer      invocation_hint G_GNUC_UNUSED,
                       gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__OBJECT_INT) (gpointer     data1,
@@ -88,10 +88,10 @@ nip_VOID__OBJECT_INT (GClosure     *closure,
 /* VOID:DOUBLE,DOUBLE (nipmarshal.list:26) */
 void
 nip_VOID__DOUBLE_DOUBLE (GClosure     *closure,
-                         GValue       *return_value,
+                         GValue       *return_value G_GNUC_UNUSED,
                          guint         n_param_values,
                          const GValue *param_values,
-                         gpointer      invocation_hint,
+                         gpointer      invocation_hint G_GNUC_UNUSED,
                          gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__DOUBLE_DOUBLE) (gpointer     data1,
@@ -122,3 +122,44 @@ nip_VOID__DOUBLE_DOUBLE (GClosure     *closure,
             data2);
 }
 
+/* BOOLEAN:INT,INT (nipmarshal.list:27) */
+void
+nip_BOOLEAN__INT_INT (GClosure     *closure,
+                      GValue       *return_value G_GNUC_UNUSED,
+                      guint         n_param_values,
+                      const GValue *param_values,
+                      gpointer      invocation_hint G_GNUC_UNUSED,
+                      gpointer      marshal_data)
+{
+  typedef gboolean (*GMarshalFunc_BOOLEAN__INT_INT) (gpointer     data1,
+                                                     gint         arg_1,
+                                                     gint         arg_2,
+                                                     gpointer     data2);
+  register GMarshalFunc_BOOLEAN__INT_INT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gboolean v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_BOOLEAN__INT_INT) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_int (param_values + 1),
+                       g_marshal_value_peek_int (param_values + 2),
+                       data2);
+
+  g_value_set_boolean (return_value, v_return);
+}
+
