@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -55,14 +55,16 @@ static ClassmodelMember number_members[] = {
 static void
 number_class_init( NumberClass *class )
 {
+	iObjectClass *iobject_class = (iObjectClass *) class;
 	ModelClass *model_class = (ModelClass *) class;
 	ClassmodelClass *classmodel_class = (ClassmodelClass *) class;
 
 	parent_class = g_type_class_peek_parent( class );
 
-
 	/* Init methods.
 	 */
+	iobject_class->user_name = _( "Number" );
+
 	model_class->view_new = number_view_new;
 
 	/* Static init.
