@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -693,26 +693,28 @@ reduce_test_char( Reduce *rc, PElement *base, int *sz )
 static gboolean
 reduce_n_is_string( Reduce *rc, PElement *base, int sz )
 {
-	reduce_spine( rc, base );
+	void *result;
 
-	if( !PEISLIST( base ) ) 
-		return( FALSE );
+	reduce_spine( rc, base );
 
 	/* We know managedstrings are strings without needing to expand them.
 	 */
 	if( PEISMANAGEDSTRING( base ) )
 		return( TRUE );
-	else {
-		void *result;
 
-		result = reduce_map_list( rc, base, 
-			(reduce_map_list_fn) reduce_test_char, &sz, NULL );
+	/* reduce_map_list() will throw an exeception if we give it a
+	 * non-list.
+	 */
+	if( !PEISLIST( base ) ) 
+		return( FALSE ); 
 
-		if( result == (void *) -1 )
-			return( FALSE );
+	result = reduce_map_list( rc, base, 
+		(reduce_map_list_fn) reduce_test_char, &sz, NULL );
 
-		return( TRUE );
-	}
+	if( result == (void *) -1 )
+		return( FALSE );
+
+	return( TRUE );
 }
 
 /* Test for object is string. Just test the first few elements, so we
@@ -1321,7 +1323,7 @@ reduce_start:
 
 		case SYM_ROOT:
 		case SYM_WORKSPACE:
-		case SYM_WORKSPACEGROUP:
+		case SYM_WORKSPACEROOT:
 			/* Becomes a symref ... base type.
 			 */
 			PEPUTP( &np, ELEMENT_SYMREF, sym );
