@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -40,7 +40,6 @@
  */
 
 /* Time row recomp.
-#define DEBUG_TIME
 #define DEBUG_TIME_SORT
  */
 
@@ -440,7 +439,8 @@ row_add_parent_name( Link *link, VipsBuf *buf )
 {
 	Row *row;
 
-	if( link->parent->expr && (row = link->parent->expr->row) ) {
+	if( link->parent->expr && 
+		(row = link->parent->expr->row) ) {
 		row_qualified_name_relative( link->child, row, buf );
 		vips_buf_appends( buf, " " );
 	}
@@ -453,7 +453,8 @@ row_add_child_name( Link *link, VipsBuf *buf )
 {
 	Row *row;
 
-	if( link->child->expr && (row = link->child->expr->row) ) {
+	if( link->child->expr && 
+		(row = link->child->expr->row) ) {
 		row_qualified_name_relative( link->parent, row, buf );
 		vips_buf_appends( buf, " " );
 	}
@@ -465,7 +466,8 @@ static void *
 row_add_dirty_child_name( Link *link, VipsBuf *buf )
 {
 	if( link->child->dirty ) {
-		symbol_qualified_name( link->child, buf );
+		symbol_qualified_name_relative( link->parent, 
+			link->child, buf );
 		vips_buf_appends( buf, " " );
 	}
 
@@ -484,9 +486,11 @@ row_info( iObject *iobject, VipsBuf *buf )
 
 	if( row->expr ) 
 		iobject_info( IOBJECT( row->expr ), buf );
-	if( row->child_rhs->itext ) 
+	if( row->child_rhs &&
+		row->child_rhs->itext ) 
 		iobject_info( IOBJECT( row->child_rhs->itext ), buf );
-	if( row->child_rhs->graphic ) 
+	if( row->child_rhs &&
+		row->child_rhs->graphic ) 
 		iobject_info( IOBJECT( row->child_rhs->graphic ), buf );
 	if( row->top_row->sym ) {
 		if( row->top_row->sym->topchildren ) {
@@ -496,7 +500,7 @@ row_info( iObject *iobject, VipsBuf *buf )
 			 */
 			vips_buf_appends( buf, _( "refers to" ) );
 			vips_buf_appends( buf, ": " );
-			slist_map( row->top_row->sym->topchildren, 
+			slist_map_rev( row->top_row->sym->topchildren, 
 				(SListMapFn) row_add_child_name, buf );
 			vips_buf_appends( buf, "\n" );
 		}
@@ -507,12 +511,14 @@ row_info( iObject *iobject, VipsBuf *buf )
 			 */
 			vips_buf_appends( buf, _( "is referred to by" ) );
 			vips_buf_appends( buf, ": " );
-			slist_map( row->top_row->sym->topparents, 
+			slist_map_rev( row->top_row->sym->topparents, 
 				(SListMapFn) row_add_parent_name, buf );
 			vips_buf_appends( buf, "\n" );
 		}
 	}
-	if( row == row->top_row && row->sym->dirty ) {
+	if( row == row->top_row && 
+		row->sym &&
+		row->sym->dirty ) {
 		Symbol *sym = row->sym;
 
 		if( sym->ndirtychildren ) {
@@ -520,7 +526,7 @@ row_info( iObject *iobject, VipsBuf *buf )
 			vips_buf_appends( buf, " " );
 			vips_buf_appends( buf, _( "is blocked on" ) );
 			vips_buf_appends( buf, ": " );
-			slist_map( sym->topchildren,
+			slist_map_rev( sym->topchildren,
 				(SListMapFn) row_add_dirty_child_name, buf );
 			vips_buf_appends( buf, "\n" );
 		}
@@ -642,7 +648,7 @@ row_scrollto( Model *model, ModelScrollPosition position )
 	 */
 	if( !col->open ) {
 		column_set_open( col, TRUE );
-		model_scrollto( MODEL( col ), position );
+		column_scrollto( col, position );
 	}
 
 	MODEL_CLASS( parent_class )->scrollto( model, position );
@@ -699,12 +705,6 @@ row_load( Model *model,
 		expr_dirty( row->sym->expr, link_serial_new() );
 	}
 
-	/* If we've loaded a toplevel into a closed column, make sure display
-	 * is off.
-	 */
-	if( scol->is_top && !scol->top_col->open )
-		model_set_display( MODEL( row ), FALSE );
-
 	return( TRUE );
 }
 
@@ -824,13 +824,15 @@ row_save_test( Model *model )
 {
 	Row *row = ROW( model );
 	Workspace *ws = row->ws;
+	Workspacegroup *wsg = workspace_get_workspacegroup( ws );
+
 	gboolean save;
 
 	if( row == row->top_row ) {
 		/* This is a top-level row ... save unless we're in
 		 * only-save-selected mode.
 		 */
-		if( ws->save_type == WORKSPACE_SAVE_SELECTED )
+		if( wsg->save_type == WORKSPACEGROUP_SAVE_SELECTED )
 			save = row->selected;
 		else
 			save = TRUE;
@@ -846,7 +848,6 @@ row_save_test( Model *model )
 				(icontainer_map_fn) row_calculate_to_save, 
 				NULL );
 		}
-
 	}
 	else 
 		save = row->to_save;
@@ -878,13 +879,7 @@ row_new_heap( Heapmodel *heapmodel, PElement *root )
 		if( is_super( row->sym ) && PEISCLASS( root ) &&
 			*IOBJECT( PEGETCLASSCOMPILE( root )->sym )->name == 
 			'_' )
-			model_set_display( MODEL( row ), FALSE );
-		/* Hide top-level rows in closed columns.
-		 */
-		else if( row->scol->is_top && !row->top_col->open )
-			model_set_display( MODEL( row ), FALSE );
-		else
-			model_set_display( MODEL( row ), TRUE );
+			model_display( MODEL( row ), FALSE );
 	}
 
 	/* New value ... reset error state.
@@ -1625,10 +1620,12 @@ row_recomp_all( Row *top_row )
 		g_timer_reset( timer );
 #endif /*DEBUG_ROW*/
 
-		if( !row_recomp_row( dirty_row ) ) 
+		if( !row_recomp_row( dirty_row ) ) {
 			/* This will set top_row->err and end the loop.
 			 */
-			expr_error_set( dirty_row->expr );
+			if( dirty_row->expr ) 
+				expr_error_set( dirty_row->expr );
+		}
 		else
 			row_dirty_clear( dirty_row );
 
@@ -1650,7 +1647,6 @@ row_recomp( Row *row )
 {
 	Row *top_row = row->top_row;
 
-#ifdef DEBUG_TIME
 	static GTimer *recomp_timer = NULL;
 
 	if( !recomp_timer )
@@ -1658,11 +1654,6 @@ row_recomp( Row *row )
 
 	g_timer_reset( recomp_timer );
 
-	printf( "row_recomp: starting for dirties on " );
-	row_name_print( top_row );
-	printf( "\n" );
-#endif /*DEBUG_TIME*/
-
 	/* Sort dirties into recomp order.
 	 */
 	row_recomp_sort( top_row );
@@ -1685,6 +1676,11 @@ row_recomp( Row *row )
 	 */
 	row_recomp_all( top_row );
 
+	/* Our workspace may have been closed in a callback: bail out.
+	 */
+	if( !top_row->sym )
+		return;
+
 	/* Add all static row links. Have to do this after any 
 	 * parsing in row_recomp_all().
 	 */
@@ -1709,14 +1705,6 @@ row_recomp( Row *row )
 	 */
 	if( top_row->sym->ndirtychildren != 0 ) {
 		IM_FREEF( g_slist_free, top_row->recomp_save );
-
-#ifdef DEBUG_TIME
-		printf( "row_recomp: delaying recomp of " );
-		row_name_print( top_row );
-		printf( "after %gs\n",  
-			g_timer_elapsed( recomp_timer, NULL ) );
-#endif /*DEBUG_TIME*/
-
 		return;
 	}
 
@@ -1743,6 +1731,11 @@ row_recomp( Row *row )
 		/* Rebuild all dirty rows in a second pass.
 		 */
 		row_recomp_all( top_row );
+
+		/* Our workspace may have been closed in a callback: bail out.
+		 */
+		if( !top_row->sym )
+			return;
 	}
 
 	IM_FREEF( g_slist_free, top_row->recomp_save );
@@ -1759,11 +1752,15 @@ row_recomp( Row *row )
 		(icontainer_map_fn) heapmodel_update_model, NULL ) )
 		expr_error_set( top_row->expr );
 
-#ifdef DEBUG_TIME
-	printf( "row_recomp: done for dirties of " );
-	row_name_print( top_row );
-	printf( "in %gs\n",  g_timer_elapsed( recomp_timer, NULL ) );
-#endif /*DEBUG_TIME*/
+	if( main_option_profile ) { 
+		char txt[100];
+		VipsBuf buf = VIPS_BUF_STATIC( txt );
+		Symbol *context = symbol_get_parent( top_row->ws->sym );
+
+		row_qualified_name_relative( context, top_row, &buf );
+		printf( "%s\t%g\n", vips_buf_all( &buf ), 
+			g_timer_elapsed( recomp_timer, NULL ) );
+	}
 
 #ifdef DEBUG
 	printf( "row_recomp: value of " );
@@ -2022,7 +2019,8 @@ row_set_status( Row *row )
 
 		vips_buf_appends( &buf, " = " );
 
-		if( row->ws->mode != WORKSPACE_MODE_FORMULA )
+		if( row->ws && 
+			row->ws->mode != WORKSPACE_MODE_FORMULA )
 			vips_buf_appends( &buf, NN( itext->formula ) );
 		else 
 			vips_buf_appends( &buf, vips_buf_all( &itext->value ) );
@@ -2071,7 +2069,7 @@ row_parse_name_row( Row *context, const char *path )
 }
 
 /* Parse a qualified name .. eg. "untitled.A1.name" and find the row. Find
- * relative to context. Context is a sym, so we can have workspacegroups etc.
+ * relative to context. Context is a sym, so we can have workspaceroot etc.
  */
 Row *
 row_parse_name( Symbol *context, const char *path )
@@ -2103,7 +2101,7 @@ row_parse_name( Symbol *context, const char *path )
 	 */
 	switch( context->type ) {
 	case SYM_WORKSPACE:
-	case SYM_WORKSPACEGROUP:
+	case SYM_WORKSPACEROOT:
 	case SYM_ROOT:
 		if( !(sym = compile_lookup( context->expr->compile, name )) ) 
 			return( NULL );
