@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -104,6 +104,26 @@ tool_linkreport_tool( Tool *tool, VipsBuf *buf, gboolean *found )
 		(symbol_map_fn) tool_linkreport_sym, buf, found ) );
 }
 
+static void
+tool_finalize( GObject *gobject )
+{
+	Tool *tool;
+
+#ifdef DEBUG
+	printf( "tool_finalize: %p %s\n", 
+		gobject, NN( IOBJECT( gobject )->name ) );
+#endif /*DEBUG*/
+
+	g_return_if_fail( gobject != NULL );
+	g_return_if_fail( IS_TOOL( gobject ) );
+
+	tool = TOOL( gobject );
+
+	IM_FREE( tool->help );
+
+	G_OBJECT_CLASS( parent_class )->finalize( gobject );
+}
+
 static void *toolitem_free( Toolitem *toolitem );
 
 /* Remove a tool. Also strip the sym, if any.
@@ -200,6 +220,10 @@ tool_type_to_char( Tooltype type )
 
 	default:
 		g_assert( FALSE );
+		
+		/* Keep gcc happy.
+		 */
+		return( FALSE );
 	}
 }
 
@@ -247,6 +271,7 @@ tool_class_init( ToolClass *class )
 
 	/* Init methods.
 	 */
+	gobject_class->finalize = tool_finalize;
 	gobject_class->dispose = tool_dispose;
 
 	iobject_class->info = tool_info;
@@ -439,37 +464,9 @@ toolitem_set_tooltip( Toolitem *toolitem, PElement *root )
 			MEMBER_TOOLTIP, value, MAX_NAME ) ) {
 		IM_SETSTR( toolitem->tooltip, _( value ) );
 	}
-	else {
-		char *p;
-
-		if( (p = toolitem->compile->text) ) {
-			/* Skip leading whitespace.
-			 */
-			while( isspace( (int)(*p) ) )
-				p++;
-
-			/* Skip leading comment, if any.
-			 */
-			if( p[0] == '/' && p[1] == '*' )
-				p += 2;
-			else if( p[0] == '/' && p[1] == '/' )
-				p += 2;
-
-			/* Skip more whitespace.
-			 */
-			while( isspace( (int)(*p) ) )
-				p++;
-
-			/* Limit to MAX_NAME chars or 1st line. Strip trailing
-			 * whitespace.
-			 */
-			im_strncpy( value, p, MAX_NAME );
-			if( (p = strchr( value, '\n' )) )
-				*p = '\0';
-			*((char *) my_strrspn( value, WHITESPACE )) = '\0';
-			IM_SETSTR( toolitem->tooltip, value );
-		}
-	}
+	else if( toolitem->tool &&
+		toolitem->tool->help ) 
+		IM_SETSTR( toolitem->tooltip, toolitem->tool->help );
 }
 
 static void
@@ -777,6 +774,53 @@ tool_new_value_cb( Symbol *sym, Tool *tool )
 	tool_toolitem_rebuild( tool );
 }
 
+static void
+tool_set_help( Tool *tool ) 
+{
+	char *p;
+	char value[MAX_NAME];
+
+	if( tool->sym &&
+		tool->sym->expr &&
+		tool->sym->expr->compile &&
+		(p = tool->sym->expr->compile->text) ) {
+		/* Skip leading whitespace.
+		 */
+		while( isspace( (int)(*p) ) )
+			p++;
+
+		/* Skip leading comment, if any.
+		 */
+		if( p[0] == '/' && p[1] == '*' )
+			p += 2;
+		else if( p[0] == '/' && p[1] == '/' )
+			p += 2;
+
+		/* Skip more whitespace.
+		 */
+		while( isspace( (int)(*p) ) )
+			p++;
+
+		/* Limit to MAX_NAME chars or 1st line. Strip trailing
+		 * whitespace.
+		 */
+		im_strncpy( value, p, MAX_NAME );
+		if( (p = strchr( value, '\n' )) )
+			*p = '\0';
+		*((char *) my_strrspn( value, WHITESPACE )) = '\0';
+
+		IM_SETSTR( tool->help, value );
+	}
+	else if( tool->sym &&
+		tool->sym->type == SYM_EXTERNAL )
+		IM_SETSTR( tool->help, tool->sym->function->desc );
+	else if( tool->sym &&
+		tool->sym->type == SYM_BUILTIN )
+		IM_SETSTR( tool->help, tool->sym->builtin->desc );
+	else
+		IM_SETSTR( tool->help, NULL );
+}
+
 /* Add a symbol to a toolkit. 
  */
 Tool *
@@ -789,9 +833,11 @@ tool_new_sym( Toolkit *kit, int pos, Symbol *sym )
 	/* Is there a tool we can reuse? Don't update pos .. assume we want to
 	 * keep the old one.
 	 */
-	if( sym->tool && sym->tool->kit == kit ) {
-		sym->tool->lineno = -1;
-		return( sym->tool );
+	if( (tool = sym->tool) &&
+		tool->kit == kit ) {
+		tool->lineno = -1;
+		tool_set_help( tool );
+		return( tool );
 	}
 
 	/* Junk any existing tool for this sym.
@@ -800,7 +846,7 @@ tool_new_sym( Toolkit *kit, int pos, Symbol *sym )
 		sym->tool = NULL;
 		tool->sym = NULL;
 
-		iobject_destroy( IOBJECT( tool ) );
+		IDESTROY( tool );
 	}
 
 	tool = TOOL( g_object_new( TYPE_TOOL, NULL ) );
@@ -812,6 +858,8 @@ tool_new_sym( Toolkit *kit, int pos, Symbol *sym )
 	tool->link_sym = sym;
 	tool_link( tool, kit, pos, IOBJECT( sym )->name );
 
+	tool_set_help( tool );
+
 #ifdef DEBUG
 	printf( "tool_new_sym: new tool for " );
 	symbol_name_print( sym );
