@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -35,9 +35,42 @@
 
 static ViewClass *parent_class = NULL;
 
-/* Link menus to workspaces with this.
+/* Link menu items to toolview with this.
  */
-static GQuark toolview_ws_quark = 0;
+static GQuark toolview_quark = 0;
+
+static Mainw *
+toolview_get_mainw( Toolview *tview )
+{
+	if( !tview->kview ||
+		!tview->kview->kitgview )
+		return( NULL );
+
+	return( tview->kview->kitgview->mainw );
+}
+
+static Workspace *
+toolview_get_workspace( Toolview *tview )
+{
+	Mainw *mainw;
+
+	if( !(mainw = toolview_get_mainw( tview )) )
+		return( NULL );
+
+	return( mainw_get_workspace( mainw ) ); 
+}
+
+static Workspace *
+item_get_workspace( GtkWidget *item )
+{
+	Toolview *tview;
+
+	if( !(tview = gtk_object_get_data_by_id( GTK_OBJECT( item ), 
+		toolview_quark )) )
+		return( NULL );
+
+	return( toolview_get_workspace( tview ) );
+}
 
 static void 
 toolview_destroy( GtkObject *object )
@@ -71,8 +104,7 @@ toolview_finalize( GObject *gobject )
 static void
 toolview_activate_cb( GtkWidget *widget, Toolitem *toolitem )
 {
-	Workspace *ws = gtk_object_get_data_by_id( GTK_OBJECT( widget ), 
-		toolview_ws_quark );
+	Workspace *ws = item_get_workspace( widget );
 
 	switch( toolitem->tool->type ) {
 	case TOOL_DIA:
@@ -99,18 +131,20 @@ toolview_activate_cb( GtkWidget *widget, Toolitem *toolitem )
 static void 
 toolview_select_cb( GtkWidget *widget, Toolitem *toolitem )
 {
-	Workspace *ws = gtk_object_get_data_by_id( GTK_OBJECT( widget ), 
-		toolview_ws_quark );
+	Workspace *ws = item_get_workspace( widget );
 
-	if( ws && toolitem->help )
+	if( ws && 
+		toolitem->help )
 		workspace_set_status( ws, "%s", toolitem->help );
 }
 
 /* Sub fn of below ... build a menu item for a TOOL_SYM. 
  */
 static GtkWidget *
-toolview_refresh_sub( Toolitem *toolitem, Workspace *ws, GtkWidget *menu )
+toolview_refresh_sub( Toolview *tview, 
+	Toolitem *toolitem, Workspace *ws, GtkWidget *menu )
 {
+	Mainw *mainw = toolview_get_mainw( tview );
 	GtkWidget *item;
 
 	if( toolitem->is_separator ) 
@@ -119,7 +153,7 @@ toolview_refresh_sub( Toolitem *toolitem, Workspace *ws, GtkWidget *menu )
 		item = gtk_image_menu_item_new_with_mnemonic( toolitem->label );
 
 		gtk_object_set_data_by_id( GTK_OBJECT( item ),
-			toolview_ws_quark, ws );
+			toolview_quark, tview );
 
 		if( toolitem->icon )
 			gtk_image_menu_item_set_image( 
@@ -131,7 +165,8 @@ toolview_refresh_sub( Toolitem *toolitem, Workspace *ws, GtkWidget *menu )
 				GTK_SIGNAL_FUNC( toolview_activate_cb ), 
 				toolitem );
 
-		if( toolitem->help )
+		if( toolitem->help && 
+			!toolitem->is_pullright )
 			set_tooltip( item, "%s", toolitem->help );
 
 		gtk_signal_connect( GTK_OBJECT( item ), "select", 
@@ -145,7 +180,7 @@ toolview_refresh_sub( Toolitem *toolitem, Workspace *ws, GtkWidget *menu )
 			GSList *p;
 
 			gtk_menu_set_accel_group( GTK_MENU( submenu ), 
-				ws->iwnd->accel_group );
+				IWINDOW( mainw )->accel_group );
 			gtk_menu_set_accel_path( GTK_MENU( submenu ), 
 				toolitem->path );
 
@@ -157,7 +192,8 @@ toolview_refresh_sub( Toolitem *toolitem, Workspace *ws, GtkWidget *menu )
 			for( p = toolitem->children; p; p = p->next ) {
 				Toolitem *child = p->data;
 
-				toolview_refresh_sub( child, ws, submenu );
+				toolview_refresh_sub( tview, 
+					child, ws, submenu );
 			}
 
 			gtk_menu_item_set_submenu( GTK_MENU_ITEM( item ), 
@@ -181,20 +217,6 @@ toolview_refresh_sub( Toolitem *toolitem, Workspace *ws, GtkWidget *menu )
 	return( item );
 }
 
-/* Get the workspace we act on.
- */
-static Workspace *
-toolview_get_workspace( Toolview *tview )
-{
-	View *view;
-
-	for( view = VIEW( tview ); !IS_TOOLKITGROUPVIEW( view ); 
-		view = view->parent ) 
-		;
-
-	return( TOOLKITGROUPVIEW( view )->mainw->ws );
-}
-
 /* Our widget has been destroyed. NULL out or pointer to it, to stop us
  * destroying it again later.
  */
@@ -221,14 +243,14 @@ toolview_refresh( vObject *vobject )
 	iobject_print( VOBJECT( tview )->iobject );
 #endif /*DEBUG*/
 
-	if( !toolview_ws_quark ) 
-		toolview_ws_quark = 
-			g_quark_from_static_string( "toolview_ws_quark" );
+	if( !toolview_quark ) 
+		toolview_quark = 
+			g_quark_from_static_string( "toolview_quark" );
 
 	DESTROY_GTK( tview->item );
 
 	if( tool->toolitem ) 
-		tview->item = toolview_refresh_sub( tool->toolitem, 
+		tview->item = toolview_refresh_sub( tview, tool->toolitem, 
 			ws, kview->menu );
 
 	if( tview->item )
