@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -229,6 +229,14 @@ set_sprop( xmlNode *xnode, const char *name, const char *value )
 	return( TRUE );
 }
 
+/* Set an xml property from an optionally NULL string.
+ */
+gboolean
+set_iprop( xmlNode *xnode, const char *name, int value )
+{
+	return( set_prop( xnode, name, "%d", value ) );
+}
+
 /* Save a list of strings. For name=="fred" and n strings in list, save as
  * "fredn" == n, "fred0" == list[0], etc.
  */
@@ -1270,7 +1278,7 @@ static const int nbandfmt_names = IM_NUMBER( bandfmt_names );
 const char *
 decode_bandfmt( int f )
 {
-	if( f > nbandfmt_names || f < 0 )
+	if( f > nbandfmt_names - 1 || f < 0 )
 		return( _( "<unknown format>" ) );
 	else
 		return( _( bandfmt_names[f] ) );
@@ -1306,14 +1314,15 @@ static const char *type_names[] = {
 	"Fourier",		/* IM_TYPE_FOURIER	24 */
 	"RGB16",		/* IM_TYPE_RGB16	25 */
 	"GREY16",		/* IM_TYPE_GREY16	26 */
-	"Array"			/* VIPS_INTERPRETATION_ARRAY = 27 */
+	"Array",		/* VIPS_INTERPRETATION_ARRAY = 27 */
+	"scRGB"			/* VIPS_INTERPRETATION_scRGB = 28 */
 };
 static const int ntype_names = IM_NUMBER( type_names );
 
 const char *
 decode_type( int t )
 {
-	if( t > ntype_names || t < 0 )
+	if( t > ntype_names - 1 || t < 0 )
 		return( _( "<unknown type>" ) );
 	else
 		return( type_names[t] );
@@ -1587,32 +1596,6 @@ absoluteize_path( char *path )
 	}
 }
 
-/* Guess VIPSHOME, if we can.
- */
-const char *
-get_vipshome( const char *argv0 )
-{
-	static char prefix_buffer[FILENAME_MAX];
-	static const char *prefix;
-
-	if( !prefix ) {
-		if( !(prefix = im_guess_prefix( argv0, "VIPSHOME" )) ) {
-			error_top( _( "Unable to find install area." ) );
-			error_vips();
-
-			return( NULL );
-		}
-
-		im_strncpy( prefix_buffer, prefix, FILENAME_MAX );
-
-		absoluteize_path( prefix_buffer );
-		canonicalize_path( prefix_buffer );
-		setenvf( "VIPSHOME", "%s", prefix_buffer );
-	}
-
-	return( prefix_buffer );
-}
-
 /* Call a void*-valued function, building a string arg. We expand env. 
  * variables, but that's all.
  */
@@ -1844,6 +1827,38 @@ isdir( const char *filename, ... )
 	return( res );
 }
 
+
+static void *
+mtime_sub( const char *filename, time_t *time )
+{
+	struct stat st;
+
+	if( stat( filename, &st ) == -1 )
+		return( NULL );
+#ifdef HAVE_GETEUID
+	if( st.st_uid != geteuid() )
+		return( NULL );
+#endif /*HAVE_GETEUID*/
+	*time = st.st_mtime;
+
+	return( NULL );
+}
+
+time_t
+mtime( const char *filename, ... )
+{
+	va_list ap;
+	time_t time;
+
+	time = 0;
+        va_start( ap, filename );
+        (void) callv_string_filenameva( 
+		(callv_string_fn) mtime_sub, filename, ap, &time, NULL, NULL );
+        va_end( ap );
+
+	return( time );
+}
+
 gboolean
 mkdirf( const char *name, ... )
 {
@@ -1852,7 +1867,7 @@ mkdirf( const char *name, ... )
 
         va_start( ap, name );
         res = calli_string_filenameva( 
-		(calli_string_fn) mkdir, 
+		(calli_string_fn) g_mkdir, 
 			name, ap, GINT_TO_POINTER( 0755 ), NULL, NULL ) == 0;
         va_end( ap );
 
@@ -1914,8 +1929,16 @@ is_absolute( const char *fname )
 	char buf[FILENAME_MAX];
 
 	expand_variables( fname, buf );
+	nativeize_path( buf );
 
-	if( g_path_is_absolute( buf ) )
+	/* We can't use g_path_is_absolute(), we might be given a Windows path
+	 * including a drive specifier, and g_path_is_absolute() on unix does
+	 * not know about Windows paths.
+	 *
+	 * We should probably look out for whitespace.
+	 */
+	if( buf[0] == '/' ||
+		(buf[0] != '\0' && buf[1] == ':') )
 		return( TRUE );
 	else
 		return( FALSE );
@@ -1999,7 +2022,7 @@ ifile_open_read( const char *name, ... )
 
 	if( !of )
 		return( NULL );
-	if( !(of->fname_real = path_find_file( PATH_SEARCH, of->fname )) ) {
+	if( !(of->fname_real = path_find_file( of->fname )) ) {
 		error_top( _( "Unable to open." ) );
 		error_sub( _( "Unable to open file \"%s\" for reading.\n%s." ),
 			of->fname, g_strerror( errno ) );
@@ -2114,6 +2137,7 @@ char *
 ifile_read( iOpenFile *of )
 {
 	long len;
+	size_t len2;
 	char *str;
 
 	/* Find length.
@@ -2131,16 +2155,14 @@ ifile_read( iOpenFile *of )
 	 */
 	if( !(str = imalloc( NULL, len + 1 )) ) 
 		return( NULL );
-	if( fread( str, sizeof( char ), (size_t) len, of->fp ) != 
-		(size_t) len ) {
-		of->last_errno = errno;
-		IM_FREE( str );
-		error_top( _( "Unable to read." ) );
-		error_sub( _( "Unable to read from file \"%s\".\n%s." ),
-			of->fname_real, g_strerror( of->last_errno ) );
-		return( NULL );
-	}
-	str[len] = '\0';
+
+	/* We can't check len2 against len, since we may be reading a text
+	 * file on Windows, in which case this fread will change CRLF to LF
+	 * and len2 will be less than len.
+	 */
+	len2 = fread( str, sizeof( char ), (size_t) len, of->fp );
+
+	str[len2] = '\0';
 
 	return( str );
 }
@@ -2700,11 +2722,6 @@ recent_add( GSList *recent, const char *filename )
 	im_strncpy( absolute, filename, FILENAME_MAX );
 	absoluteize_path( absolute );
 
-	/* Bit of a hack ... handy place to make sure this directory is on the
-	 * session path.
-	 */
-	path_add_file( absolute );
-
 	for( p = recent; p; p = p->next ) {
 		const char *stored = p->data;
 
@@ -2811,7 +2828,7 @@ recent_save( GSList *recent, const char *filename )
 	recent_free( old_recent );
 }
 
-/* Return the name of the save dir we use ... eg. "/home/john/.nip2-7.10.8",
+/* Return the name of the save dir we use ... eg. "$HOME/.nip2-7.10.8",
  * or maybe "C:\Documents and Settings\john\Application Data"
  */
 const char *
