@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -41,9 +41,9 @@ static ViewClass *parent_class = NULL;
 
 /* Params for "Align Columns" function.
  */
-static const int workspaceview_layout_snap_threshold = 20;
+static const int workspaceview_layout_snap_threshold = 30;
 static const int workspaceview_layout_hspacing = 10;
-static const int workspaceview_layout_vspacing = 30;
+static const int workspaceview_layout_vspacing = 10;
 static const int workspaceview_layout_left = WORKSPACEVIEW_MARGIN_LEFT;
 static const int workspaceview_layout_top = WORKSPACEVIEW_MARGIN_TOP;
 
@@ -128,7 +128,7 @@ workspaceview_scroll_update( Workspaceview *wview )
 	ws->vp = wview->vp;
 
 #ifdef DEBUG
-	printf( "workspaceview_scroll_update:\n" );
+	printf( "workspaceview_scroll_update: %s\n", IOBJECT( ws )->name );
 	printf( "  wview->vp: l=%d, t=%d, w=%d, h=%d; fixed w=%d; h=%d\n",
 		wview->vp.left, wview->vp.top, 
 		wview->vp.width, wview->vp.height,
@@ -411,7 +411,7 @@ workspaceview_destroy( GtkObject *object )
 	Workspaceview *wview;
 
 #ifdef DEBUG
-	printf( "workspaceview_destroy\n" );
+	printf( "workspaceview_destroy: %p\n", object );
 #endif /*DEBUG*/
 
 	g_return_if_fail( object != NULL );
@@ -424,6 +424,7 @@ workspaceview_destroy( GtkObject *object )
 	workspaceview_scroll_stop( wview );
 	IM_FREEF( iwindow_cursor_context_destroy, wview->context );
 	FREESID( wview->watch_changed_sid, main_watchgroup );
+	DESTROY_GTK( wview->popup );
 
 	GTK_OBJECT_CLASS( parent_class )->destroy( object );
 }
@@ -431,6 +432,15 @@ workspaceview_destroy( GtkObject *object )
 static void
 workspaceview_realize( GtkWidget *widget )
 {
+#ifdef DEBUG
+{
+	Workspaceview *wview = WORKSPACEVIEW( widget );
+	Workspace *ws = WORKSPACE( VOBJECT( wview )->iobject );
+
+	printf( "workspaceview_realize: %s\n", IOBJECT( ws )->name );
+}
+#endif /*DEBUG*/
+
 	GTK_WIDGET_CLASS( parent_class )->realize( widget );
 
 	/* Mark us as a symbol drag-to widget. 
@@ -461,43 +471,38 @@ workspaceview_drag_data_received( GtkWidget *widget, GdkDragContext *context,
 		selection_data->format == 8 &&
 		workspaceview_is_background( wview, 
 			GTK_WIDGET( wview->fixed )->window, x, y ) &&
-		(from_row = row_parse_name( main_workspacegroup->sym, 
+		(from_row = row_parse_name( main_workspaceroot->sym, 
 			from_row_path )) ) {
-		char *name;
+		char new_name[MAX_STRSIZE];
 		Column *col;
+		char vips_buf_text[256];
+		VipsBuf buf = VIPS_BUF_STATIC( vips_buf_text );
+		Symbol *sym;
 
-		name = workspace_column_name_new( ws, NULL );
-		if( !(col = column_new( ws, name )) )
+		workspace_column_name_new( ws, new_name );
+		col = column_new( ws, new_name );
+
+		col->x = x;
+		col->y = y;
+		workspace_column_select( ws, col );
+
+		/* Qualify relative to us. We don't want to embed
+		 * workspace names unless we have to.
+		 */
+		row_qualified_name_relative( ws->sym, from_row, &buf );
+
+		if( !(sym = workspace_add_def( ws, vips_buf_all( &buf ) )) ) 
 			iwindow_alert( widget, GTK_MESSAGE_ERROR );
-		IM_FREE( name );
 
-		if( col ) {
-			char vips_buf_text[256];
-			VipsBuf buf = VIPS_BUF_STATIC( vips_buf_text );
-			Symbol *sym;
+		symbol_recalculate_all();
 
-			col->x = x;
-			col->y = y;
-			workspace_column_select( ws, col );
-
-			/* Qualify relative to us. We don't want to embed
-			 * workspace names unless we have to.
-			 */
-			row_qualified_name_relative( ws->sym, 
-				from_row, &buf );
-
-			if( !(sym = workspace_add_def( ws, 
-				vips_buf_all( &buf ) )) ) 
-				iwindow_alert( widget, GTK_MESSAGE_ERROR );
-
-			symbol_recalculate_all();
-
-			/* Usually the drag-from row will be selected, very
-			 * annoying. Select the drag-to row.
-			 */
-			if( sym && sym->expr && sym->expr->row )
-				row_select( sym->expr->row );
-		}
+		/* Usually the drag-from row will be selected, very
+		 * annoying. Select the drag-to row.
+		 */
+		if( sym && 
+			sym->expr && 
+			sym->expr->row )
+			row_select( sym->expr->row );
 	}
 }
 
@@ -524,6 +529,7 @@ workspaceview_child_size_cb( Columnview *cview,
 	GtkAllocation *allocation, Workspaceview *wview )
 {
 	Workspace *ws = WORKSPACE( VOBJECT( wview )->iobject );
+	Workspacegroup *wsg = workspace_get_workspacegroup( ws );
 
 	int right, bottom;
 
@@ -540,16 +546,21 @@ workspaceview_child_size_cb( Columnview *cview,
 		(view_map_fn) workspaceview_child_size_sub, 
 		&wview->bounding, NULL );
 
-	wview->bounding.width += 3;
-	wview->bounding.height += 30;
+	wview->bounding.width += 1000;
+	wview->bounding.height += 1000;
 
 #ifdef DEBUG
-	printf( "workspaceview_child_size_cb: "
+{
+	Column *col = COLUMN( VOBJECT( cview )->iobject );
+
+	printf( "workspaceview_child_size_cb: cview %s "
 		"bb left=%d, top=%d, width=%d, height=%d\n",
+		IOBJECT( col )->name, 
 		wview->bounding.left,
 		wview->bounding.top,
 		wview->bounding.width,
 		wview->bounding.height );
+}
 #endif /*DEBUG*/
 
 	/* Resize our fixed if necessary.
@@ -557,15 +568,61 @@ workspaceview_child_size_cb( Columnview *cview,
 	right = IM_RECT_RIGHT( &wview->bounding );
 	bottom = IM_RECT_BOTTOM( &wview->bounding );
 	if( right != wview->width || bottom != wview->height ) {
+		gtk_widget_set_size_request( GTK_WIDGET( wview->fixed ), 
+			right, bottom ); 
+
 		/* Update the model hints ... it uses bounding to position
 		 * loads and saves.
 		 */
 		ws->area = wview->bounding;
-		filemodel_set_offset( FILEMODEL( ws ), 
+		filemodel_set_offset( FILEMODEL( wsg ), 
 			ws->area.left, ws->area.top );
 	}
 }
 
+/* Pick an xy position for the next column.
+ */
+static void
+workspaceview_pick_xy( Workspaceview *wview, int *x, int *y )
+{
+	/* Position already set? No change.
+	 */
+	if( *x >= 0 )
+		return;
+
+	/* Set this position.
+	 */
+	*x = wview->next_x + wview->vp.left;
+	*y = wview->next_y + wview->vp.top;
+
+	/* And move on.
+	 */
+	wview->next_x += 30;
+	wview->next_y += 30;
+	if( wview->next_x > 300 )
+		wview->next_x = 3;
+	if( wview->next_y > 200 )
+		wview->next_y = 3;
+}
+
+static void
+workspaceview_link( View *view, Model *model, View *parent )
+{
+	Workspaceview *wview = WORKSPACEVIEW( view );
+	Workspace *ws = WORKSPACE( model );
+
+	VIEW_CLASS( parent_class )->link( view, model, parent );
+
+	vobject_link( VOBJECT( wview->toolkitbrowser ), 
+		IOBJECT( ws->kitg ) );
+	vobject_link( VOBJECT( wview->workspacedefs ), IOBJECT( ws ) );
+
+	toolkitbrowser_set_workspace( wview->toolkitbrowser, ws ); 
+
+	pane_set_state( wview->rpane, ws->rpane_open, ws->rpane_position ); 
+	pane_set_state( wview->lpane, ws->lpane_open, ws->lpane_position ); 
+}
+
 static void
 workspaceview_child_add( View *parent, View *child )
 {
@@ -585,19 +642,6 @@ workspaceview_child_add( View *parent, View *child )
 		GTK_WIDGET( cview ), column->x, column->y );
 	cview->lx = column->x;
 	cview->ly = column->y;
-	wview->front = cview;
-}
-
-static void
-workspaceview_child_remove( View *parent, View *child )
-{
-	Columnview *cview = COLUMNVIEW( child );
-	Workspaceview *wview = WORKSPACEVIEW( parent );
-
-	if( wview->front == cview )
-		wview->front = NULL;
-
-	VIEW_CLASS( parent_class )->child_remove( parent, child );
 }
 
 static void
@@ -618,25 +662,60 @@ workspaceview_child_front( View *parent, View *child )
 	Workspaceview *wview = WORKSPACEVIEW( parent );
 	Columnview *cview = COLUMNVIEW( child );
 
-	if( wview->front != cview ) {
 		gtk_widget_ref( GTK_WIDGET( cview ) );
 		gtk_container_remove( GTK_CONTAINER( wview->fixed ),
 			GTK_WIDGET( cview ) );
 		gtk_fixed_put( GTK_FIXED( wview->fixed ),
 			GTK_WIDGET( cview ), cview->lx, cview->ly );
 		gtk_widget_unref( GTK_WIDGET( cview ) );
-
-		wview->front = cview;
-	}
 }
 
 static void 
 workspaceview_refresh( vObject *vobject )
 {
+	Workspaceview *wview = WORKSPACEVIEW( vobject );
+	Workspace *ws = WORKSPACE( VOBJECT( wview )->iobject );
+
 #ifdef DEBUG
-	printf( "workspaceview_refresh\n" );
+	printf( "workspaceview_refresh: %p %s\n", ws, IOBJECT( ws )->name );
 #endif /*DEBUG*/
 
+	gtk_widget_set_sensitive( GTK_WIDGET( wview ), !ws->locked );
+
+	workspace_jump_update( ws, wview->popup_jump );
+
+	if( ws->rpane_open && !wview->rpane->open )
+		pane_animate_open( wview->rpane );
+	if( !ws->rpane_open && wview->rpane->open )
+		pane_animate_closed( wview->rpane );
+
+	if( ws->lpane_open && !wview->lpane->open )
+		pane_animate_open( wview->lpane );
+	if( !ws->lpane_open && wview->lpane->open )
+		pane_animate_closed( wview->lpane );
+
+	if( wview->label ) {
+		gtk_label_set_text( GTK_LABEL( wview->label ),
+			IOBJECT( ws )->name );
+
+		if( IOBJECT( ws )->caption )
+			set_tooltip( wview->label, 
+				"%s", IOBJECT( ws )->caption );
+
+		if( ws->locked ) 
+			gtk_image_set_from_stock( GTK_IMAGE( wview->padlock ), 
+				STOCK_LOCK, GTK_ICON_SIZE_MENU );
+		else
+			gtk_image_clear( GTK_IMAGE( wview->padlock ) );  
+
+		if( ws->errors ) 
+			gtk_image_set_from_stock( GTK_IMAGE( wview->alert ), 
+				STOCK_ALERT, GTK_ICON_SIZE_MENU );
+		else
+			gtk_image_clear( GTK_IMAGE( wview->alert ) );  
+
+	}
+
 	VOBJECT_CLASS( parent_class )->refresh( vobject );
 }
 
@@ -659,7 +738,7 @@ typedef struct _WorkspaceLayout {
 	 */
 	int out_x, out_y;
 
-	/* Accumulate areas here.
+	/* Accumulate the size of the current set of columns here.
 	 */
 	Rect area;
 
@@ -692,10 +771,29 @@ static void *
 workspaceview_layout_find_similar_x( Columnview *cview, 
 	WorkspaceLayout *layout )
 {
-	if( ABS( GTK_WIDGET( cview )->allocation.x - layout->area.left ) < 
-		workspaceview_layout_snap_threshold ) 
+	int x = GTK_WIDGET( cview )->allocation.x;
+
+	gboolean snap;
+
+	snap = FALSE;
+
+	/* Special case: a colum at zero makes a new column on the far left.
+	 */
+	if( layout->area.left == 0 && 
+		x == 0 ) 
+		snap = TRUE;
+
+	if( layout->area.left > 0 &&
+		ABS( x - layout->area.left ) < 
+			workspaceview_layout_snap_threshold ) 
+		snap = TRUE;
+	
+	if( snap ) { 
 		layout->current_columns = g_slist_prepend(
 			layout->current_columns, cview );
+		layout->area.width = IM_MAX( layout->area.width, 
+			GTK_WIDGET( cview )->allocation.width ); 
+	}
 
 	return( NULL );
 }
@@ -713,16 +811,35 @@ workspaceview_layout_set_pos( Columnview *cview, WorkspaceLayout *layout )
 {
 	Column *column = COLUMN( VOBJECT( cview )->iobject );
 
-	column->x = layout->out_x;
-	column->y = layout->out_y;
+	gboolean changed;
+
+	changed = FALSE;
+
+	/* If this column is being dragged, put the xy we allocate into the
+	 * shadow instead. 
+	 */
+	if( cview->shadow ) {
+		if( cview->shadow->lx != layout->out_x ||
+			cview->shadow->ly != layout->out_y ) {
+			cview->shadow->lx = layout->out_x;
+			cview->shadow->ly = layout->out_y;
+			changed = TRUE;
+		}
+	}
+	else {
+		if( column->x != layout->out_x ||
+			column->y != layout->out_y ) { 
+			column->x = layout->out_x;
+			column->y = layout->out_y;
+			changed = TRUE;
+		}
+	}
+
 	layout->out_y += GTK_WIDGET( cview )->allocation.height +
 		workspaceview_layout_vspacing;
 
-	if( GTK_WIDGET( cview )->allocation.width > layout->area.width )
-		layout->area.width = GTK_WIDGET( cview )->allocation.width;
-
-	iobject_changed( IOBJECT( column ) );
-	filemodel_set_modified( FILEMODEL( column->ws ), TRUE );
+	if( changed ) 
+		iobject_changed( IOBJECT( column ) ); 
 
 	return( NULL );
 }
@@ -745,6 +862,7 @@ workspaceview_layout_loop( WorkspaceLayout *layout )
 		(SListMapFn) workspaceview_layout_find_leftmost, layout );
 
 	layout->current_columns = NULL;
+	layout->area.width = GTK_WIDGET( layout->cview )->allocation.width;
 	slist_map( layout->undone_columns,
 		(SListMapFn) workspaceview_layout_find_similar_x, layout );
 
@@ -752,7 +870,6 @@ workspaceview_layout_loop( WorkspaceLayout *layout )
 		(GCompareFunc) workspaceview_layout_sort_y );
 
 	layout->out_y = workspaceview_layout_top;
-	layout->area.width = 0;
 	slist_map( layout->current_columns,
 		(SListMapFn) workspaceview_layout_set_pos, layout );
 
@@ -761,18 +878,16 @@ workspaceview_layout_loop( WorkspaceLayout *layout )
 	slist_map( layout->current_columns,
 		(SListMapFn) workspaceview_layout_strike, layout );
 
-	g_slist_free( layout->current_columns );
-	layout->current_columns = NULL;
+	IM_FREEF( g_slist_free, layout->current_columns );
 }
 
 /* Autolayout ... try to rearrange columns so they don't overlap. 
 
 	Strategy:
 
-	search for top-left-most column
+	search for left-most column
 
-	search for all columns with a similar X (ie. within a smallish
-	threshold)
+	search for all columns with a 'small' overlap
 
 	lay those columns out vertically with some space between them ... keep
 	the vertical ordering we had before
@@ -816,33 +931,145 @@ workspaceview_class_init( WorkspaceviewClass *class )
 
 	vobject_class->refresh = workspaceview_refresh;
 
+	view_class->link = workspaceview_link;
 	view_class->child_add = workspaceview_child_add;
-	view_class->child_remove = workspaceview_child_remove;
 	view_class->child_position = workspaceview_child_position;
 	view_class->child_front = workspaceview_child_front;
 	view_class->layout = workspaceview_layout;
 }
 
+/* Can't use main_load(), we want to select wses after load.
+ */
+static gboolean
+workspaceview_load( Workspace *ws, const char *filename )
+{
+	Workspacegroup *wsg = workspace_get_workspacegroup( ws );
+	Workspaceroot *wsr = wsg->wsr; 
+
+	Workspacegroup *new_wsg;
+
+	if( (new_wsg = mainw_open_workspace( wsr, filename )) ) 
+		return( TRUE );
+
+	error_clear();
+
+	/* workspace_load_file() needs to recalc to work, try to avoid that by
+	 * doing .defs first.
+	 */
+	if( is_file_type( &filesel_dfile_type, filename ) ) {
+		if( toolkit_new_from_file( main_toolkitgroup, filename ) )
+			return( TRUE );
+
+		error_clear();
+	}
+
+	/* Try as matrix or image. Have to do these via definitions.
+	 */
+	if( workspace_load_file( ws, filename ) ) 
+		return( TRUE );
+
+	error_clear();
+
+	error_top( _( "Unknown file type." ) );
+	error_sub( _( "Unable to load \"%s\"." ), filename );
+
+	return( FALSE );
+}
+
+static void
+workspaceview_lpane_changed_cb( Pane *pane, Workspaceview *wview )
+{
+	Workspace *ws;
+
+	if( (ws = WORKSPACE( VOBJECT( wview )->iobject )) ) 
+		if( ws->lpane_open != pane->open ||
+			ws->lpane_position != pane->user_position ) {
+			ws->lpane_open = pane->open;
+			ws->lpane_position = pane->user_position;
+
+			prefs_set( "WORKSPACE_LPANE_OPEN", "%d", 
+				ws->lpane_open ); 
+			prefs_set( "WORKSPACE_LPANE_POSITION", "%d", 
+				ws->lpane_position ); 
+
+			iobject_changed( IOBJECT( ws ) );
+		}
+}
+
+static void
+workspaceview_rpane_changed_cb( Pane *pane, Workspaceview *wview )
+{
+	Workspace *ws;
+
+	if( (ws = WORKSPACE( VOBJECT( wview )->iobject )) ) 
+		if( ws->rpane_open != pane->open ||
+			ws->rpane_position != pane->user_position ) {
+			ws->rpane_open = pane->open;
+			ws->rpane_position = pane->user_position;
+
+			prefs_set( "WORKSPACE_RPANE_OPEN", "%d", 
+				ws->rpane_open ); 
+			prefs_set( "WORKSPACE_RPANE_POSITION", "%d", 
+				ws->rpane_position ); 
+
+			iobject_changed( IOBJECT( ws ) );
+		}
+}
+
 static gboolean
 workspaceview_filedrop( Workspaceview *wview, const char *filename )
 {
 	Workspace *ws = WORKSPACE( VOBJECT( wview )->iobject );
+
 	gboolean result;
 
-	result = main_load( ws, filename );
+	result = workspaceview_load( ws, filename );
 	if( result )
 		symbol_recalculate_all();
 
 	return( result );
 }
 
+static void
+workspaceview_column_new_action_cb2( GtkWidget *wid, GtkWidget *host, 
+	Workspaceview *wview )
+{
+	Workspace *ws = WORKSPACE( VOBJECT( wview )->iobject );
+
+	if( !workspace_column_new( ws ) ) 
+		iwindow_alert( GTK_WIDGET( wview ), GTK_MESSAGE_ERROR );
+}
+
+static void
+workspaceview_group_action_cb2( GtkWidget *wid, GtkWidget *host, 
+	Workspaceview *wview )
+{
+	Workspace *ws = WORKSPACE( VOBJECT( wview )->iobject );
+
+	workspace_selected_group( ws );
+}
+
+static void
+workspaceview_next_error_action_cb2( GtkWidget *wid, GtkWidget *host, 
+	Workspaceview *wview )
+{
+	Workspace *ws = WORKSPACE( VOBJECT( wview )->iobject );
+
+	if( !workspace_next_error( ws ) ) {
+		error_top( _( "No errors in tab." ) );
+		error_sub( "%s", _( "There are no errors (that I can see) "
+			"in this tab." ) );
+		iwindow_alert( GTK_WIDGET( wview ), GTK_MESSAGE_INFO );
+	}
+}
+
 static void
 workspaceview_init( Workspaceview *wview )
 {
 	GtkAdjustment *hadj;
 	GtkAdjustment *vadj;
-
-	wview->wgview = NULL;
+	Panechild *panechild;
+	GtkWidget *ebox;
 
 	wview->fixed = NULL;
 	wview->window = NULL;
@@ -869,18 +1096,33 @@ workspaceview_init( Workspaceview *wview )
 	wview->next_x = 3;
 	wview->next_y = 3;
 
-	wview->front = NULL;
-
 	wview->context = NULL; 
 
 	wview->watch_changed_sid = g_signal_connect( main_watchgroup, 
 		"watch_changed",
 		G_CALLBACK( workspaceview_watch_changed_cb ), wview );
 
+	wview->rpane = pane_new( PANE_HIDE_RIGHT );
+	g_signal_connect( wview->rpane, "changed",
+		G_CALLBACK( workspaceview_rpane_changed_cb ), wview );
+	gtk_box_pack_start( GTK_BOX( wview ), 
+		GTK_WIDGET( wview->rpane ), TRUE, TRUE, 2 );
+	gtk_widget_show( GTK_WIDGET( wview->rpane ) );
+
+	wview->lpane = pane_new( PANE_HIDE_LEFT );
+	g_signal_connect( wview->lpane, "changed",
+		G_CALLBACK( workspaceview_lpane_changed_cb ), wview );
+	gtk_paned_pack1( GTK_PANED( wview->rpane ), 
+		GTK_WIDGET( wview->lpane ), TRUE, FALSE );
+	gtk_widget_show( GTK_WIDGET( wview->lpane ) );
+
 	/* Ask for our own window so we can spot events on the window 
 	 * background.
 	 */
 	wview->fixed = gtk_fixed_new();
+	gtk_widget_add_events( GTK_WIDGET( wview->fixed ), 
+		GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK |
+		GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK ); 
 	gtk_fixed_set_has_window( GTK_FIXED( wview->fixed ), TRUE );
 	wview->window = gtk_scrolled_window_new( NULL, NULL );
 	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( wview->window ), 
@@ -919,11 +1161,51 @@ workspaceview_init( Workspaceview *wview )
          * ourselves .. see rowview.c.
          */
 
-	gtk_box_pack_end( GTK_BOX( wview ), wview->window, TRUE, TRUE, 0 );
+	gtk_paned_pack2( GTK_PANED( wview->lpane ), 
+		GTK_WIDGET( wview->window ), TRUE, FALSE );
+
+	/* Toolkit Browser pane.
+	 */
+	panechild = panechild_new( wview->rpane, _( "Toolkit Browser" ) );
+
+	/* Have to put toolkitbrowser in an ebox so the search entry gets
+	 * clipped to the pane size.
+	 */
+	ebox = gtk_event_box_new();
+	gtk_container_add( GTK_CONTAINER( panechild ), GTK_WIDGET( ebox ) );
+	gtk_widget_show( ebox );
+
+	wview->toolkitbrowser = toolkitbrowser_new();
+	gtk_container_add( GTK_CONTAINER( ebox ), 
+		GTK_WIDGET( wview->toolkitbrowser ) );
+	gtk_widget_show( GTK_WIDGET( wview->toolkitbrowser ) );
+
+	/* Workspace-local defs pane.
+	 */
+	panechild = panechild_new( wview->lpane, _( "Tab Definitions" ) );
+
+	wview->workspacedefs = workspacedefs_new();
+	gtk_container_add( GTK_CONTAINER( panechild ), 
+		GTK_WIDGET( wview->workspacedefs ) );
+	gtk_widget_show( GTK_WIDGET( wview->workspacedefs ) );
 
 	filedrop_register( GTK_WIDGET( wview ),
 		(FiledropFunc) workspaceview_filedrop, wview );
 
+	wview->popup = popup_build( _( "Workspace menu" ) );
+
+	popup_add_but( wview->popup, _( "New C_olumn" ),
+		POPUP_FUNC( workspaceview_column_new_action_cb2 ) ); 
+	wview->popup_jump = popup_add_pullright( wview->popup, 
+		_( "Jump to _Column" ) ); 
+	menu_add_sep( wview->popup );
+	popup_add_but( wview->popup, _( "_Group Selected" ),
+		POPUP_FUNC( workspaceview_group_action_cb2 ) ); 
+	menu_add_sep( wview->popup );
+	popup_add_but( wview->popup, STOCK_NEXT_ERROR,
+		POPUP_FUNC( workspaceview_next_error_action_cb2 ) ); 
+	popup_attach( wview->fixed, wview->popup, wview );
+
 	gtk_widget_show_all( wview->window );
 }
 
@@ -958,28 +1240,15 @@ workspaceview_new( void )
 	return( VIEW( wview ) );
 }
 
-/* Pick an xy position for the next column.
- */
 void
-workspaceview_pick_xy( Workspaceview *wview, int *x, int *y )
+workspaceview_set_label( Workspaceview *wview, 
+	GtkWidget *label, GtkWidget *padlock, GtkWidget *alert )
 {
-	/* Position already set? No change.
-	 */
-	if( *x >= 0 )
-		return;
+	g_assert( !wview->label );
+	g_assert( !wview->padlock );
+	g_assert( !wview->alert );
 
-	/* Set this position.
-	 */
-	*x = wview->next_x + wview->vp.left;
-	*y = wview->next_y + wview->vp.top;
-
-	/* And move on.
-	 */
-	wview->next_x += 30;
-	wview->next_y += 30;
-	if( wview->next_x > 300 )
-		wview->next_x = 3;
-	if( wview->next_y > 200 )
-		wview->next_y = 3;
+	wview->label = label;
+	wview->padlock = padlock;
+	wview->alert = alert;
 }
-
