@@ -1,8 +1,11 @@
 #!/bin/bash
 
+# set -x
+
 top_srcdir=$1
 tmp=$top_srcdir/test/tmp
-image=$top_srcdir/share/nip2/data/examples/businesscard/slanted_oval_vase2.jpg
+test_images=$top_srcdir/test/images
+image=$test_images/slanted_oval_vase2.jpg
 chain=1
 
 # im_benchmark needs a labq
@@ -35,6 +38,7 @@ done
 
 if [[ $max > 0 ]]; then
 	echo error, max == $max
+	exit 1
 else
 	echo all threading tests passed
 fi
